@@ -365,13 +365,16 @@ class GeckoWebExtension(
                 creatorUrl = it.creatorUrl,
                 reviewUrl = it.reviewUrl,
                 version = it.version,
-                permissions = it.promptPermissions.toList(),
+                requiredPermissions = it.requiredPermissions.toList(),
+                // Origins is marked as @NonNull but may be null: https://bugzilla.mozilla.org/show_bug.cgi?id=1629957
+                requiredOrigins = it.requiredOrigins.orEmpty().toList(),
+                requiredDataCollectionPermissions = it.requiredDataCollectionPermissions.toList(),
                 optionalPermissions = it.optionalPermissions.toList(),
+                optionalOrigins = it.optionalOrigins.toList(),
+                optionalDataCollectionPermissions = it.optionalDataCollectionPermissions.toList(),
                 grantedOptionalPermissions = it.grantedOptionalPermissions.toList(),
                 grantedOptionalOrigins = it.grantedOptionalOrigins.toList(),
-                optionalOrigins = it.optionalOrigins.toList(),
-                // Origins is marked as @NonNull but may be null: https://bugzilla.mozilla.org/show_bug.cgi?id=1629957
-                hostPermissions = it.origins.orEmpty().toList(),
+                grantedOptionalDataCollectionPermissions = it.grantedOptionalDataCollectionPermissions.toList(),
                 disabledFlags = DisabledFlags.select(it.disabledFlags),
                 optionsPageUrl = it.optionsPageUrl,
                 openOptionsPageInTab = it.openOptionsPageInTab,
@@ -403,6 +406,13 @@ class GeckoWebExtension(
     internal fun getIcon(size: Int): GeckoResult<Bitmap> {
         return nativeExtension.metaData.icon.getBitmap(size)
     }
+
+    /**
+     * Companion object for [GeckoWebExtension].
+     */
+    companion object {
+        val DATA_COLLECTION_PERMISSIONS: List<String> = GeckoNativeWebExtension.DATA_COLLECTION_PERMISSIONS
+    }
 }
 
 /**
