@@ -28,7 +28,8 @@ import mozilla.components.browser.state.state.UndoHistoryState
 import mozilla.components.browser.state.state.WebExtensionState
 import mozilla.components.browser.state.state.content.DownloadState
 import mozilla.components.browser.state.state.content.FindResultState
-import mozilla.components.browser.state.state.content.ShareInternetResourceState
+import mozilla.components.browser.state.state.content.PermissionHighlightsState
+import mozilla.components.browser.state.state.content.ShareResourceState
 import mozilla.components.browser.state.state.extension.WebExtensionPromptRequest
 import mozilla.components.browser.state.state.recover.RecoverableTab
 import mozilla.components.browser.state.state.recover.TabState
@@ -48,6 +49,7 @@ import mozilla.components.concept.engine.search.SearchRequest
 import mozilla.components.concept.engine.translate.Language
 import mozilla.components.concept.engine.translate.LanguageModel
 import mozilla.components.concept.engine.translate.LanguageSetting
+import mozilla.components.concept.engine.translate.ModelManagementOptions
 import mozilla.components.concept.engine.translate.TranslationDownloadSize
 import mozilla.components.concept.engine.translate.TranslationEngineState
 import mozilla.components.concept.engine.translate.TranslationError
@@ -510,6 +512,28 @@ sealed class ContentAction : BrowserAction() {
         data class MediaKeySystemAccesChangedAction(val tabId: String, val value: Boolean) :
             UpdatePermissionHighlightsStateAction()
 
+        /**
+         * Updates the [PermissionHighlightsState.localDeviceAccessChanged] property with the
+         * given [tabId]
+         *
+         * @property tabId The affected tab id
+         * @property value The value indicating whether or not the local device access permission
+         * has changed
+         */
+        data class LocalDeviceAccessChangedAction(val tabId: String, val value: Boolean) :
+            UpdatePermissionHighlightsStateAction()
+
+        /**
+         * Updates the [PermissionHighlightsState.localNetworkAccessChanged] property with the
+         * given [tabId]
+         *
+         * @property tabId The affected tab id
+         * @property value The value indicating whether or not the local network access permission
+         * has changed
+         */
+        data class LocalNetworkAccessChangedAction(val tabId: String, val value: Boolean) :
+            UpdatePermissionHighlightsStateAction()
+
         /**
          * Updates the autoPlayAudibleChanged property of the [PermissionHighlightsState]
          * with the given [tabId].
@@ -809,7 +833,7 @@ sealed class ContentAction : BrowserAction() {
     /**
      * Updates the [ContentState] of the given [sessionId] to indicate whether or not desktop mode is enabled.
      */
-    data class UpdateDesktopModeAction(val sessionId: String, val enabled: Boolean) : ContentAction()
+    data class UpdateTabDesktopMode(val sessionId: String, val enabled: Boolean) : ContentAction()
 
     /**
      * Updates the [AppIntentState] of the [ContentState] with the given [sessionId].
@@ -831,7 +855,11 @@ sealed class ContentAction : BrowserAction() {
      * Updates the [ContentState] with the provided [tabId] to the appropriate priority based on any
      * existing form data.
      */
-    data class UpdateHasFormDataAction(val tabId: String, val containsFormData: Boolean) : ContentAction()
+    data class UpdateHasFormDataAction(
+        val tabId: String,
+        val containsFormData: Boolean,
+        val adjustPriority: Boolean = true,
+    ) : ContentAction()
 
     /**
      * Lowers priority of the [tabId] to default after certain period of time
@@ -844,11 +872,17 @@ sealed class ContentAction : BrowserAction() {
     data class CheckForFormDataExceptionAction(val tabId: String, val throwable: Throwable) : ContentAction()
 
     /**
-     * Updates the [ContentState.isProductUrl] state for the non private tab with the given [tabId].
+     * Inform that the tab with [tabId] started rendering a pdf.
      */
-    data class UpdateProductUrlStateAction(
+    data class EnteredPdfViewer(
+        val tabId: String,
+    ) : ContentAction()
+
+    /**
+     * Inform that the tab with [tabId] stopped rendering a pdf.
+     */
+    data class ExitedPdfViewer(
         val tabId: String,
-        val isProductUrl: Boolean,
     ) : ContentAction()
 }
 
@@ -994,13 +1028,15 @@ sealed class TranslationsAction : BrowserAction() {
     /**
      * Indicates that the given [operation] data should be fetched for the given [tabId].
      *
-     * @property tabId The ID of the tab the [EngineSession] should be linked to.
+     * @property tabId The ID of the tab the [EngineSession] should be linked to. May be null
+     * to complete the operation on the current tab (when a tab is required for the operation)
+     * or when no session is associated with the request.
      * @property operation The translation operation that failed.
      */
     data class OperationRequestedAction(
-        override val tabId: String,
+        val tabId: String?,
         val operation: TranslationOperation,
-    ) : TranslationsAction(), ActionWithTab
+    ) : TranslationsAction()
 
     /**
      * Sets whether the device architecture supports translations or not on
@@ -1033,6 +1069,19 @@ sealed class TranslationsAction : BrowserAction() {
         val pageSettings: TranslationPageSettings?,
     ) : TranslationsAction(), ActionWithTab
 
+    /**
+     * Indicates the translation processing state on the given [tabId].
+     *
+     * A translation is processing when the engine is actively working on performing the translation.
+     *
+     * @property tabId The ID of the tab the [EngineSession] should be linked to.
+     * @property isProcessing Whether the translation is processing or not.
+     */
+    data class SetTranslateProcessingAction(
+        override val tabId: String,
+        val isProcessing: Boolean,
+    ) : TranslationsAction(), ActionWithTab
+
     /**
      * Updates the specified page setting operation on the translation engine and ensures the final
      * state on the given [tabId]'s store remains in-sync.
@@ -1121,6 +1170,16 @@ sealed class TranslationsAction : BrowserAction() {
     data class SetLanguageModelsAction(
         val languageModels: List<LanguageModel>,
     ) : TranslationsAction()
+
+    /**
+     * Manages the language machine learning translation models the translation engine has available.
+     * Has options for downloading and deleting models.
+     *
+     * @property options The operation to perform to manage the model.
+     */
+    data class ManageLanguageModelsAction(
+        val options: ModelManagementOptions,
+    ) : TranslationsAction()
 }
 
 /**
@@ -1301,6 +1360,7 @@ sealed class EngineAction : BrowserAction() {
         val flags: EngineSession.LoadUrlFlags = EngineSession.LoadUrlFlags.none(),
         val additionalHeaders: Map<String, String>? = null,
         val includeParent: Boolean = false,
+        val textDirectiveUserActivation: Boolean = false,
     ) : EngineAction(), ActionWithTab
 
     /**
@@ -1664,19 +1724,28 @@ sealed class DownloadAction : BrowserAction() {
      * Restores the given [download] from the storage.
      */
     data class RestoreDownloadStateAction(val download: DownloadState) : DownloadAction()
+
+    /**
+     * [BrowserAction] to remove completed and cancelled downloads from the storage that no longer
+     * exist on disk.
+     *
+     * This action is used to clean up the download storage by removing entries for files
+     * that have been deleted or moved from their original download location.
+     */
+    data object RemoveDeletedDownloads : DownloadAction()
 }
 
 /**
- * [BrowserAction] implementations related to updating the session state of internet resources to be shared.
+ * [BrowserAction] implementations related to updating the session state of internet or local resources to be shared.
  */
-sealed class ShareInternetResourceAction : BrowserAction() {
+sealed class ShareResourceAction : BrowserAction() {
     /**
-     * Starts the sharing process of an Internet resource.
+     * Starts the sharing process of an Internet or local resource.
      */
     data class AddShareAction(
         val tabId: String,
-        val internetResource: ShareInternetResourceState,
-    ) : ShareInternetResourceAction()
+        val resource: ShareResourceState,
+    ) : ShareResourceAction()
 
     /**
      * Previous share request is considered completed.
@@ -1685,7 +1754,7 @@ sealed class ShareInternetResourceAction : BrowserAction() {
      */
     data class ConsumeShareAction(
         val tabId: String,
-    ) : ShareInternetResourceAction()
+    ) : ShareResourceAction()
 }
 
 /**
@@ -1697,7 +1766,7 @@ sealed class CopyInternetResourceAction : BrowserAction() {
      */
     data class AddCopyAction(
         val tabId: String,
-        val internetResource: ShareInternetResourceState,
+        val internetResource: ShareResourceState.InternetResource,
     ) : CopyInternetResourceAction()
 
     /**
@@ -1766,6 +1835,11 @@ sealed class SearchAction : BrowserAction() {
      */
     data class SetRegionAction(val regionState: RegionState, val distribution: String? = null) : SearchAction()
 
+    /**
+     * Application Search Engines have finished loading from disk.
+     */
+    data class ApplicationSearchEnginesLoaded(val applicationSearchEngines: List<SearchEngine>) : SearchAction()
+
     /**
      * Sets the list of search engines and default search engine IDs.
      */
@@ -1839,6 +1913,11 @@ sealed class SearchAction : BrowserAction() {
     object RestoreHiddenSearchEnginesAction : SearchAction()
 }
 
+/**
+ * [BrowserAction] implements setting and updating the distribution
+ */
+data class UpdateDistribution(val distributionId: String?) : BrowserAction()
+
 /**
  * [BrowserAction] implementations for updating state needed for debugging. These actions should
  * be carefully considered before being used.
@@ -1873,3 +1952,18 @@ sealed class AppLifecycleAction : BrowserAction() {
      */
     object PauseAction : AppLifecycleAction()
 }
+
+/**
+ * [BrowserAction] implementations related to updating the application's default desktop mode setting.
+ */
+sealed class DefaultDesktopModeAction : BrowserAction() {
+    /**
+     * Toggles the global default for desktop browsing mode.
+     */
+    data object ToggleDesktopMode : DefaultDesktopModeAction()
+
+    /**
+     * Updates the global default for desktop browsing mode.
+     */
+    data class DesktopModeUpdated(val newValue: Boolean) : DefaultDesktopModeAction()
+}
