@@ -11,6 +11,8 @@ import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHig
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.AutoPlayInAudibleBlockingAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.AutoPlayInAudibleChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.CameraChangedAction
+import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.LocalDeviceAccessChangedAction
+import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.LocalNetworkAccessChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.LocationChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.MediaKeySystemAccesChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.MicrophoneChangedAction
@@ -33,10 +35,15 @@ import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.HitResult
 import mozilla.components.concept.engine.history.HistoryItem
 import mozilla.components.concept.engine.manifest.WebAppManifest
+import mozilla.components.concept.engine.permission.Permission.AppLocationCoarse
+import mozilla.components.concept.engine.permission.Permission.ContentGeoLocation
+import mozilla.components.concept.engine.permission.PermissionRequest
 import mozilla.components.concept.engine.prompt.PromptRequest
 import mozilla.components.concept.engine.window.WindowRequest
 import mozilla.components.support.test.ext.joinBlocking
+import mozilla.components.support.test.libstate.ext.waitUntilIdle
 import mozilla.components.support.test.mock
+import mozilla.components.support.test.whenever
 import org.junit.Assert.assertEquals
 import org.junit.Assert.assertFalse
 import org.junit.Assert.assertNotEquals
@@ -47,6 +54,7 @@ import org.junit.Before
 import org.junit.Test
 import org.junit.runner.RunWith
 import org.mockito.Mockito.spy
+import org.mockito.Mockito.verify
 
 @RunWith(AndroidJUnit4::class)
 class ContentActionTest {
@@ -787,6 +795,24 @@ class ContentActionTest {
         assertTrue(tab.content.permissionHighlights.mediaKeySystemAccessChanged)
     }
 
+    @Test
+    fun `WHEN dispatching LocalDeviceAccessChangedAction THEN localDeviceAccessChanged state will be updated`() {
+        assertFalse(tab.content.permissionHighlights.localDeviceAccessChanged)
+
+        store.dispatch(LocalDeviceAccessChangedAction(tab.id, true)).joinBlocking()
+
+        assertTrue(tab.content.permissionHighlights.localDeviceAccessChanged)
+    }
+
+    @Test
+    fun `WHEN dispatching LocalNetworkAccessChangedAction THEN localNetworkAccessChanged state will be updated`() {
+        assertFalse(tab.content.permissionHighlights.localNetworkAccessChanged)
+
+        store.dispatch(LocalNetworkAccessChangedAction(tab.id, true)).joinBlocking()
+
+        assertTrue(tab.content.permissionHighlights.localNetworkAccessChanged)
+    }
+
     @Test
     fun `WHEN dispatching AutoPlayAudibleChangedAction THEN autoPlayAudibleChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.autoPlayAudibleChanged)
@@ -891,4 +917,40 @@ class ContentActionTest {
 
         assertFalse(tab.content.hasFormData)
     }
+
+    @Test
+    fun `merge permission request if same request`() {
+        val url = "https://www.mozilla.org"
+
+        val request1: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(ContentGeoLocation(id = "permission")))
+            whenever(uri).thenReturn(url)
+        }
+        val request2: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(ContentGeoLocation(id = "permission")))
+            whenever(uri).thenReturn(url)
+        }
+
+        store.dispatch(ContentAction.UpdatePermissionsRequest(tab.id, request1))
+        store.dispatch(ContentAction.UpdatePermissionsRequest(tab.id, request2))
+        store.waitUntilIdle()
+
+        verify(request1).merge(request2)
+    }
+
+    @Test
+    fun `merge app permission request if same request`() {
+        val request1: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(AppLocationCoarse(id = "permission")))
+        }
+        val request2: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(AppLocationCoarse(id = "permission")))
+        }
+
+        store.dispatch(ContentAction.UpdateAppPermissionsRequest(tab.id, request1))
+        store.dispatch(ContentAction.UpdateAppPermissionsRequest(tab.id, request2))
+        store.waitUntilIdle()
+
+        verify(request1).merge(request2)
+    }
 }
