@@ -8,7 +8,6 @@ import android.content.Context
 import androidx.lifecycle.LifecycleOwner
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import kotlinx.coroutines.CoroutineScope
-import kotlinx.coroutines.ExperimentalCoroutinesApi
 import kotlinx.coroutines.Job
 import kotlinx.coroutines.launch
 import kotlinx.coroutines.test.runTest
@@ -119,7 +118,6 @@ internal open class TestableFxaAccountManager(
 const val EXPECTED_AUTH_STATE = "goodAuthState"
 const val UNEXPECTED_AUTH_STATE = "badAuthState"
 
-@ExperimentalCoroutinesApi // for runTest
 @RunWith(AndroidJUnit4::class)
 class FxaAccountManagerTest {
 
@@ -1181,7 +1179,7 @@ class FxaAccountManagerTest {
         }
 
         lateinit var waitFor: Job
-        `when`(mockAccount.getProfile(ignoreCache = false)).then {
+        `when`(mockAccount.getProfile(ignoreCache = anyBoolean())).then {
             // Hit an auth error.
             waitFor = CoroutineScope(coroutineContext).launch { manager.encounteredAuthError("a test") }
             null
@@ -1241,7 +1239,7 @@ class FxaAccountManagerTest {
         }
 
         lateinit var waitFor: Job
-        `when`(mockAccount.getProfile(ignoreCache = false)).then {
+        `when`(mockAccount.getProfile(ignoreCache = anyBoolean())).then {
             // Hit an auth error.
             waitFor = CoroutineScope(coroutineContext).launch { manager.encounteredAuthError("a test") }
             null
@@ -1323,7 +1321,7 @@ class FxaAccountManagerTest {
             }
         }
         // Upon recovery, we'll hit an 'ignore cache' path.
-        `when`(mockAccount.getProfile(ignoreCache = true)).thenReturn(profile)
+        `when`(mockAccount.getProfile(ignoreCache = anyBoolean())).thenReturn(profile)
 
         val accountObserver: AccountObserver = mock()
 
@@ -1369,7 +1367,7 @@ class FxaAccountManagerTest {
         `when`(constellation.finalizeDevice(any(), any())).thenReturn(ServiceResult.Ok)
         doAnswer {
             throw FxaPanicException("500")
-        }.`when`(mockAccount).getProfile(ignoreCache = false)
+        }.`when`(mockAccount).getProfile(ignoreCache = anyBoolean())
         `when`(mockAccount.beginOAuthFlow(any(), any())).thenReturn(testAuthFlowUrl())
         `when`(mockAccount.completeOAuthFlow(anyString(), anyString())).thenReturn(true)
         // There's no account at the start.
@@ -1503,7 +1501,7 @@ class FxaAccountManagerTest {
             System.currentTimeMillis() + 1000 * 10,
         )
 
-        `when`(mockAccount.getProfile(ignoreCache = false)).thenReturn(profile)
+        `when`(mockAccount.getProfile(ignoreCache = anyBoolean())).thenReturn(profile)
         `when`(mockAccount.beginOAuthFlow(any(), any())).thenReturn(testAuthFlowUrl(entrypoint = "home-menu"))
         `when`(mockAccount.beginPairingFlow(anyString(), any(), any())).thenReturn(testAuthFlowUrl(entrypoint = "home-menu"))
         `when`(mockAccount.completeOAuthFlow(anyString(), anyString())).thenReturn(true)
@@ -1538,7 +1536,7 @@ class FxaAccountManagerTest {
         accountObserver: AccountObserver,
         coroutineContext: CoroutineContext,
     ): FxaAccountManager {
-        `when`(mockAccount.getProfile(ignoreCache = false)).thenReturn(profile)
+        `when`(mockAccount.getProfile(ignoreCache = anyBoolean())).thenReturn(profile)
         `when`(mockAccount.deviceConstellation()).thenReturn(mock())
         `when`(mockAccount.beginOAuthFlow(any(), any())).thenReturn(null)
         `when`(mockAccount.beginPairingFlow(anyString(), any(), any())).thenReturn(null)
