@@ -153,7 +153,12 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
         get() = getNonAutocompleteText(text)
 
     override val originalText: String
-        get() = text.subSequence(0, autoCompletePrefixLength).toString()
+        get() = try {
+            text.subSequence(0, autoCompletePrefixLength).toString()
+        } catch (_: IndexOutOfBoundsException) {
+            autoCompletePrefixLength = text.length
+            text.toString()
+        }
 
     /**
      * The background color used for the autocomplete suggestion.
@@ -175,7 +180,7 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
 
     private val inputMethodManger get() = context.getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager?
 
-    @SuppressWarnings("ReturnCount")
+    @Suppress("FunctionParameterNaming", "ReturnCount", "UnusedParameter")
     private val onKeyPreIme = fun (_: View, keyCode: Int, event: KeyEvent): Boolean {
         // We only want to process one event per tap
         if (event.action != KeyEvent.ACTION_DOWN) {
@@ -200,6 +205,7 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
         return false
     }
 
+    @Suppress("FunctionParameterNaming", "UnusedParameter")
     private val onKey = fun (_: View, keyCode: Int, event: KeyEvent): Boolean {
         if (keyCode == KeyEvent.KEYCODE_ENTER || keyCode == KeyEvent.KEYCODE_NUMPAD_ENTER) {
             if (event.action != KeyEvent.ACTION_DOWN) {
@@ -463,6 +469,7 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
             addAutocompleteText(result)
         }
 
+        @Suppress("DEPRECATION")
         announceForAccessibility(text.toString())
     }
 
