@@ -23,7 +23,14 @@ import androidx.test.ext.junit.rules.ActivityScenarioRule
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
 import androidx.test.platform.app.InstrumentationRegistry
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import org.hamcrest.Matchers.closeTo
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.greaterThan
+import org.hamcrest.Matchers.hasItem
+import org.hamcrest.Matchers.lessThan
+import org.hamcrest.Matchers.not
+import org.hamcrest.Matchers.notNullValue
+import org.hamcrest.Matchers.startsWith
 import org.junit.After
 import org.junit.Assume.assumeThat
 import org.junit.Before
@@ -42,6 +49,10 @@ import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.WithDisplay
 const val DISPLAY_WIDTH = 480
 const val DISPLAY_HEIGHT = 640
 
+// Constants from {@link AccessibilityNodeInfo.java}
+const val VIRTUAL_DESCENDANT_ID_MASK = -0x100000000L
+const val VIRTUAL_DESCENDANT_ID_SHIFT = 32
+
 @RunWith(AndroidJUnit4::class)
 @MediumTest
 @WithDisplay(width = DISPLAY_WIDTH, height = DISPLAY_HEIGHT)
@@ -55,12 +66,21 @@ class AccessibilityTest : BaseSessionTest() {
     @get:Rule
     override val rules: RuleChain = RuleChain.outerRule(activityRule).around(sessionRule)
 
-    // Given a child ID, return the virtual descendent ID.
-    private fun getVirtualDescendantId(childId: Long): Int {
+    /**
+     * Given a child ID, return the virtual descendent ID.
+     *
+     * @param accessibilityNodeId The id of an AccessibilityNodeInfo.
+     */
+    private fun getVirtualDescendantId(accessibilityNodeId: Long): Int {
         try {
-            val getVirtualDescendantIdMethod =
-                AccessibilityNodeInfo::class.java.getMethod("getVirtualDescendantId", Long::class.java)
-            val virtualDescendantId = getVirtualDescendantIdMethod.invoke(null, childId) as Int
+            // Rewritten in Kotlin from https://cs.android.com/android/platform/superproject/main/+/main:frameworks/base/core/java/android/view/accessibility/AccessibilityNodeInfo.java;l=1068
+            // due to being a restricted API beginning in Android 12 and unavailable for reflection.
+            val virtualDescendantId =
+                (
+                    (accessibilityNodeId and VIRTUAL_DESCENDANT_ID_MASK)
+                        shr VIRTUAL_DESCENDANT_ID_SHIFT
+                    ).toInt()
+
             return if (virtualDescendantId == Int.MAX_VALUE) -1 else virtualDescendantId
         } catch (ex: Exception) {
             return 0
