@@ -11,16 +11,20 @@ import android.content.ClipboardManager
 import android.content.Context
 import android.os.Build
 import android.os.SystemClock
-import android.view.*
+import android.view.InputDevice
+import android.view.MotionEvent
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
 import androidx.test.platform.app.InstrumentationRegistry
-import org.hamcrest.Matchers.*
+import org.hamcrest.Matchers.arrayContainingInAnyOrder
+import org.hamcrest.Matchers.endsWith
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.startsWith
 import org.junit.Assert.assertNull
 import org.junit.Assume.assumeThat
 import org.junit.Test
 import org.junit.runner.RunWith
-import org.mozilla.geckoview.*
+import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.GeckoSession.ContentDelegate
 import org.mozilla.geckoview.GeckoSession.ContentDelegate.ContextElement
 import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate
@@ -218,6 +222,10 @@ class ContentDelegateChildTest : BaseSessionTest() {
     @WithDisplay(width = 100, height = 100)
     @Test
     fun requestContextMenuOnBlobBuffered() {
+        // Bug 1981579
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+            assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
+        }
         mainSession.loadTestPath(CONTEXT_MENU_BLOB_BUFFERED_HTML_PATH)
         mainSession.waitForPageStop()
         mainSession.waitForRoundTrip()
