@@ -4,9 +4,13 @@
 
 package org.mozilla.geckoview.test
 
+import android.os.Build
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import org.hamcrest.Matchers.closeTo
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.greaterThanOrEqualTo
+import org.hamcrest.Matchers.notNullValue
 import org.junit.After
 import org.junit.Assume.assumeThat
 import org.junit.Before
@@ -68,7 +72,6 @@ class MediaSessionTest : BaseSessionTest() {
         sessionRule.setPrefsUntilTestEnd(
             mapOf(
                 "media.mediacontrol.stopcontrol.aftermediaends" to false,
-                "dom.media.mediasession.enabled" to true,
             ),
         )
     }
@@ -79,9 +82,6 @@ class MediaSessionTest : BaseSessionTest() {
 
     @Test
     fun domMetadataPlayback() {
-        // TODO: needs bug 1700243
-        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-
         val onActivatedCalled = arrayOf(GeckoResult<Void>())
         val onMetadataCalled = arrayOf(
             GeckoResult<Void>(),
@@ -89,6 +89,8 @@ class MediaSessionTest : BaseSessionTest() {
             GeckoResult<Void>(),
             GeckoResult<Void>(),
             GeckoResult<Void>(),
+            GeckoResult<Void>(),
+            GeckoResult<Void>(),
         )
         val onPlayCalled = arrayOf(
             GeckoResult<Void>(),
@@ -97,6 +99,7 @@ class MediaSessionTest : BaseSessionTest() {
             GeckoResult<Void>(),
             GeckoResult<Void>(),
             GeckoResult<Void>(),
+            GeckoResult<Void>(),
         )
         val onPauseCalled = arrayOf(
             GeckoResult<Void>(),
@@ -105,7 +108,9 @@ class MediaSessionTest : BaseSessionTest() {
             GeckoResult<Void>(),
             GeckoResult<Void>(),
             GeckoResult<Void>(),
+            GeckoResult<Void>(),
         )
+        val onStopCalled = arrayOf(GeckoResult<Void>())
 
         // Test:
         // 1. Load DOM Media Session page which contains 3 audio tracks.
@@ -176,6 +181,20 @@ class MediaSessionTest : BaseSessionTest() {
             onPauseCalled[4],
         )
 
+        // 10. Play track 2 again.
+        //    a. Ensure onMetadata (2) is called.
+        //    b. Ensure onPlay (2) is called.
+        val completedStep10 = GeckoResult.allOf(
+            onMetadataCalled[5],
+            onPlayCalled[5],
+        )
+
+        // 10. Stop current track (2).
+        //    a. Ensure onStop (2) is called.
+        val completedStep11 = GeckoResult.allOf(
+            onStopCalled[0],
+        )
+
         val path = MEDIA_SESSION_DOM1_PATH
         val session1 = sessionRule.createOpenSession()
 
@@ -193,7 +212,7 @@ class MediaSessionTest : BaseSessionTest() {
                 mediaSession1 = mediaSession
             }
 
-            @AssertCalled(false)
+            @AssertCalled(count = 1)
             override fun onDeactivated(
                 session: GeckoSession,
                 mediaSession: MediaSession,
@@ -219,7 +238,7 @@ class MediaSessionTest : BaseSessionTest() {
                 )
             }
 
-            @AssertCalled(count = 5, order = [2])
+            @AssertCalled(count = 7, order = [2])
             override fun onMetadata(
                 session: GeckoSession,
                 mediaSession: MediaSession,
@@ -235,6 +254,7 @@ class MediaSessionTest : BaseSessionTest() {
                             DOM_META[1].title,
                             DOM_META[2].title,
                             DOM_META[1].title,
+                            DOM_META[1].title,
                         ),
                     ),
                 )
@@ -248,6 +268,7 @@ class MediaSessionTest : BaseSessionTest() {
                             DOM_META[1].artist,
                             DOM_META[2].artist,
                             DOM_META[1].artist,
+                            DOM_META[1].artist,
                         ),
                     ),
                 )
@@ -261,6 +282,7 @@ class MediaSessionTest : BaseSessionTest() {
                             DOM_META[1].album,
                             DOM_META[2].album,
                             DOM_META[1].album,
+                            DOM_META[1].album,
                         ),
                     ),
                 )
@@ -287,9 +309,9 @@ class MediaSessionTest : BaseSessionTest() {
                 )
 
                 assertThat(
-                    "Playback rate should match",
+                    "Playback rate should exist",
                     state.playbackRate,
-                    closeTo(1.0, 0.01),
+                    notNullValue(),
                 )
 
                 assertThat(
@@ -299,7 +321,7 @@ class MediaSessionTest : BaseSessionTest() {
                 )
             }
 
-            @AssertCalled(count = 5, order = [2])
+            @AssertCalled(count = 6, order = [2])
             override fun onPlay(
                 session: GeckoSession,
                 mediaSession: MediaSession,
@@ -316,6 +338,11 @@ class MediaSessionTest : BaseSessionTest() {
                 onPauseCalled[sessionRule.currentCall.counter - 1]
                     .complete(null)
             }
+
+            @AssertCalled(count = 1)
+            override fun onStop(session: GeckoSession, mediaSession: MediaSession) {
+                onStopCalled[0].complete(null)
+            }
         })
 
         sessionRule.waitForResult(completedStep2)
@@ -344,13 +371,16 @@ class MediaSessionTest : BaseSessionTest() {
 
         sessionRule.waitForResult(completedStep8b)
         sessionRule.waitForResult(completedStep9)
+        mediaSession1!!.play()
+
+        sessionRule.waitForResult(completedStep10)
+        mediaSession1!!.stop()
+
+        sessionRule.waitForResult(completedStep11)
     }
 
     @Test
     fun defaultMetadataPlayback() {
-        // TODO: needs bug 1700243
-        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-
         val onActivatedCalled = arrayOf(GeckoResult<Void>())
         val onPlayCalled = arrayOf(
             GeckoResult<Void>(),
@@ -445,9 +475,6 @@ class MediaSessionTest : BaseSessionTest() {
 
     @Test
     fun domMultiSessions() {
-        // TODO: needs bug 1700243
-        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-
         val onActivatedCalled = arrayOf(
             arrayOf(GeckoResult<Void>()),
             arrayOf(GeckoResult<Void>()),
@@ -588,9 +615,9 @@ class MediaSessionTest : BaseSessionTest() {
                 )
 
                 assertThat(
-                    "Playback rate should match",
+                    "Playback rate should exist",
                     state.playbackRate,
-                    closeTo(1.0, 0.01),
+                    notNullValue(),
                 )
 
                 assertThat(
@@ -790,9 +817,10 @@ class MediaSessionTest : BaseSessionTest() {
 
     @Test
     fun fullscreenVideoElementMetadata() {
-        // TODO: bug 1810736
-        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-
+        // Bug 1981579
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+            assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
+        }
         sessionRule.setPrefsUntilTestEnd(
             mapOf(
                 "media.autoplay.default" to 0,
@@ -956,9 +984,10 @@ class MediaSessionTest : BaseSessionTest() {
 
     @Test
     fun fullscreenVideoWithActivated() {
-        // TODO: bug 1810736
-        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-
+        // Bug 1981579
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+            assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
+        }
         sessionRule.setPrefsUntilTestEnd(
             mapOf(
                 "media.autoplay.default" to 0,
@@ -999,9 +1028,10 @@ class MediaSessionTest : BaseSessionTest() {
 
     @Test
     fun switchingProcess() {
-        // TODO: bug 1810736
-        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-
+        // Bug 1981579
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+            assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
+        }
         sessionRule.setPrefsUntilTestEnd(
             mapOf(
                 "media.autoplay.default" to 0,
