@@ -5,22 +5,27 @@
 package org.mozilla.geckoview.test
 
 import android.content.ClipDescription
-import android.net.Uri
 import android.os.Build
 import android.os.Handler
 import android.os.Looper
 import android.os.SystemClock
 import android.text.InputType
+import android.text.SpannableString
+import android.text.SpannedString
 import android.view.KeyEvent
 import android.view.View
+import android.view.inputmethod.BaseInputConnection
 import android.view.inputmethod.EditorInfo
 import android.view.inputmethod.ExtractedTextRequest
 import android.view.inputmethod.InputConnection
 import android.view.inputmethod.InputContentInfo
+import androidx.core.net.toUri
 import androidx.test.filters.MediumTest
 import androidx.test.filters.SdkSuppress
 import androidx.test.platform.app.InstrumentationRegistry
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.not
+import org.hamcrest.Matchers.notNullValue
 import org.junit.Assume.assumeThat
 import org.junit.Ignore
 import org.junit.Test
@@ -155,14 +160,48 @@ class TextInputDelegateTest : BaseSessionTest() {
             ic.commitText(text, newCursorPosition)
             return
         }
+        // InputConnection.commitText might not dispatch composition event if no composition.
+        // So we look both input event and compositionend event for the completion.
         val promise = mainSession.evaluatePromiseJS(
             when (id) {
-                "#designmode" -> "new Promise(r => document.querySelector('$id').contentDocument.addEventListener('compositionend', r, { once: true }))"
-                else -> "new Promise(r => document.querySelector('$id').addEventListener('compositionend', r, { once: true }))"
+                "#designmode" -> """(function() {
+                      const doc = document.querySelector('$id').contentDocument;
+                      const p1 = new Promise(r => doc.addEventListener('compositionend', r, { once: true }));
+                      const p2 = new Promise(
+                          r => doc.addEventListener('input',
+                              function handler(e) {
+                                if (!e.isComposing) {
+                                  r();
+                                  doc.removeEventListener('input', handler);
+                                }
+                              }));
+                      return Promise.any([p1, p2]);
+                    })()
+                """.trimIndent()
+                else -> """(function() {
+                      const element = document.querySelector('$id');
+                      const p1 = new Promise(r => element.addEventListener('compositionend', r, { once: true }));
+                      const p2 = new Promise(
+                          r => element.addEventListener('input',
+                              function handler(e) {
+                                if (!e.isComposing) {
+                                  r();
+                                  element.removeEventListener('input', handler);
+                                }
+                              }));
+                      return Promise.any([p1, p2]);
+                    })()
+                """.trimIndent()
             },
         )
         ic.commitText(text, newCursorPosition)
         promise.value
+
+        // In Gecko, commit text always set caret position to the end of committed text.
+        // So if the newCursorPosition isn't 1, Gecko may notify new position after committing text.
+        if (newCursorPosition != 1) {
+            mainSession.waitForJS("new Promise(r => requestAnimationFrame(() => requestAnimationFrame(r)))")
+        }
     }
 
     private fun deleteSurroundingText(ic: InputConnection, before: Int, after: Int) {
@@ -193,6 +232,13 @@ class TextInputDelegateTest : BaseSessionTest() {
         promise.value
     }
 
+    private fun pressKeyNoWait(ic: InputConnection, keyCode: Int) {
+        val time = SystemClock.uptimeMillis()
+        val keyEvent = KeyEvent(time, time, KeyEvent.ACTION_DOWN, keyCode, 0)
+        ic.sendKeyEvent(keyEvent)
+        ic.sendKeyEvent(KeyEvent.changeAction(keyEvent, KeyEvent.ACTION_UP))
+    }
+
     private fun pressKey(ic: InputConnection, keyCode: Int) {
         val promise = mainSession.evaluatePromiseJS(
             when (id) {
@@ -200,10 +246,7 @@ class TextInputDelegateTest : BaseSessionTest() {
                 else -> "new Promise(r => document.querySelector('$id').addEventListener('keyup', r, { once: true }))"
             },
         )
-        val time = SystemClock.uptimeMillis()
-        val keyEvent = KeyEvent(time, time, KeyEvent.ACTION_DOWN, keyCode, 0)
-        ic.sendKeyEvent(keyEvent)
-        ic.sendKeyEvent(KeyEvent.changeAction(keyEvent, KeyEvent.ACTION_UP))
+        pressKeyNoWait(ic, keyCode)
         promise.value
     }
 
@@ -211,6 +254,9 @@ class TextInputDelegateTest : BaseSessionTest() {
         // Workaround for sync shadow text
         ic.beginBatchEdit()
         ic.endBatchEdit()
+
+        processChildEvents()
+        processParentEvents()
     }
 
     @Test fun restartInput() {
@@ -375,6 +421,49 @@ class TextInputDelegateTest : BaseSessionTest() {
         })
     }
 
+    @Test fun restartInput_disableEnable() {
+        assumeThat("input only", id, equalTo("#input"))
+
+        mainSession.textInput.view = View(InstrumentationRegistry.getInstrumentation().targetContext)
+        mainSession.loadTestPath(FORMS_HTML_PATH)
+        mainSession.waitForPageStop()
+
+        mainSession.evaluateJS(
+            """
+            document.querySelector('#tel1').value = "123-45"
+            document.querySelector('#tel1').focus()
+            """.trimIndent(),
+        )
+        mainSession.waitUntilCalled(GeckoSession.TextInputDelegate::class, "restartInput")
+
+        val ic = mainSession.textInput.onCreateInputConnection(EditorInfo())!!
+
+        mainSession.delegateDuringNextWait(object : TextInputDelegate {
+            @AssertCalled(count = 0)
+            override fun hideSoftInput(session: GeckoSession) {
+            }
+        })
+
+        var promise = mainSession.evaluatePromiseJS(
+            """
+            new Promise(r =>
+                document.querySelector('#tel1').addEventListener('input', e => {
+                    const element = document.querySelector('#tel1');
+                    element.setAttribute('disabled', 'disabled');
+                    element.value = "123-456";
+                    element.removeAttribute('disabled');
+                    element.focus();
+                    window.setTimeout(r, 100);
+                }, { once: true }));
+            """.trimIndent(),
+        )
+
+        pressKeyNoWait(ic, KeyEvent.KEYCODE_0)
+        promise.value
+
+        assertThat("hideSoftInput isn't called", true, equalTo(true))
+    }
+
     private fun getText(ic: InputConnection) =
         ic.getExtractedText(ExtractedTextRequest(), 0).text.toString()
 
@@ -680,6 +769,7 @@ class TextInputDelegateTest : BaseSessionTest() {
 
     @WithDisplay(width = 512, height = 512)
     // Child process updates require having a display.
+    @Ignore("Failing frequently, see: https://bugzilla.mozilla.org/show_bug.cgi?id=1741790")
     @Test
     fun inputConnection_selectionByArrowKey() {
         setupContent("")
@@ -880,7 +970,7 @@ class TextInputDelegateTest : BaseSessionTest() {
 
         // InputContentInfo requires content:// uri, so we have to set test data to custom content provider.
         TestContentProvider.setTestData(this.getTestBytes("/assets/www/images/test.gif"), "image/gif")
-        val info = InputContentInfo(Uri.parse("content://org.mozilla.geckoview.test.provider/gif"), ClipDescription("test", arrayOf("image/gif")))
+        val info = InputContentInfo("content://org.mozilla.geckoview.test.provider/gif".toUri(), ClipDescription("test", arrayOf("image/gif")))
         ic.commitContent(info, 0, null)
         promise.value
         assertThat("Input event is fired by inserting image", true, equalTo(true))
@@ -1262,6 +1352,83 @@ class TextInputDelegateTest : BaseSessionTest() {
         }
     }
 
+    @WithDisplay(width = 512, height = 512)
+    // Child process updates require having a display.
+    @Test
+    fun editorInfo_autocorrect() {
+        // design mode is always on.
+        assumeThat("Not in designmode", id, not(equalTo("#designmode")))
+
+        sessionRule.setPrefsUntilTestEnd(mapOf("dom.forms.autocorrect" to true))
+
+        mainSession.textInput.view = View(InstrumentationRegistry.getInstrumentation().targetContext)
+
+        mainSession.loadTestPath(INPUTS_PATH)
+        mainSession.waitForPageStop()
+
+        textContent = ""
+        val values = listOf("on", "off")
+        for (autocorrect in values) {
+            mainSession.evaluateJS(
+                """
+                document.querySelector('$id').setAttribute('autocorrect', '$autocorrect');
+                document.querySelector('$id').focus()""",
+            )
+            mainSession.waitUntilCalled(GeckoSession.TextInputDelegate::class, "restartInput")
+
+            val editorInfo = EditorInfo()
+            mainSession.textInput.onCreateInputConnection(editorInfo)
+            assertThat(
+                "EditorInfo.inputType by $autocorrect",
+                editorInfo.inputType and InputType.TYPE_TEXT_FLAG_AUTO_CORRECT,
+                equalTo(
+                    when (autocorrect) {
+                        "on" -> InputType.TYPE_TEXT_FLAG_AUTO_CORRECT
+                        else -> 0
+                    },
+                ),
+            )
+
+            mainSession.evaluateJS("document.querySelector('$id').blur()")
+            mainSession.waitUntilCalled(GeckoSession.TextInputDelegate::class, "restartInput")
+        }
+    }
+
+    @WithDisplay(width = 512, height = 512)
+    // Child process updates require having a display.
+    @Test
+    fun editorInfo_dynamic() {
+        // no way on designmode
+        assumeThat("Not in designmode", id, not(equalTo("#designmode")))
+
+        mainSession.textInput.view = View(InstrumentationRegistry.getInstrumentation().targetContext)
+
+        mainSession.loadTestPath(INPUTS_PATH)
+        mainSession.waitForPageStop()
+        textContent = ""
+
+        mainSession.evaluateJS(
+            """
+            document.querySelector('$id').setAttribute('inputmode', 'text');
+            document.querySelector('$id').focus()""",
+        )
+        mainSession.waitUntilCalled(GeckoSession.TextInputDelegate::class, "restartInput")
+
+        mainSession.setHandlingUserInput(true)
+        mainSession.evaluateJS("document.querySelector('$id').setAttribute('inputmode', 'numeric')")
+        mainSession.setHandlingUserInput(false)
+        mainSession.waitUntilCalled(GeckoSession.TextInputDelegate::class, "restartInput")
+
+        val editorInfo = EditorInfo()
+        mainSession.textInput.onCreateInputConnection(editorInfo)
+
+        assertThat(
+            "EditorInfo.inputType by dynamic change",
+            editorInfo.inputType and InputType.TYPE_NUMBER_VARIATION_NORMAL,
+            equalTo(InputType.TYPE_NUMBER_VARIATION_NORMAL),
+        )
+    }
+
     @WithDisplay(width = 512, height = 512)
     // Child process updates require having a display.
     @Test
@@ -1296,14 +1463,40 @@ class TextInputDelegateTest : BaseSessionTest() {
         setupContent("")
         val ic = mainSession.textInput.onCreateInputConnection(EditorInfo())!!
 
+        val promise = mainSession.evaluatePromiseJS(
+            when (id) {
+                "#designmode" -> """
+                      new Promise(
+                          r => document.querySelector('$id').contentDocument.addEventListener('keyup', function handler(e) {
+                                if (e.key == "c") {
+                                  r();
+                                  document.querySelector('$id').contentDocument.removeEventListener('keyup', handler);
+                                }
+                              }))
+                """.trimIndent()
+
+                else -> """
+                      new Promise(
+                          r => document.querySelector('$id').addEventListener('keyup', function handler(e) {
+                                if (e.key == "c") {
+                                  r();
+                                  document.querySelector('$id').removeEventListener('keyup', handler);
+                                }
+                              }))
+                """.trimIndent()
+            },
+        )
+
         // Emulate GBoard's InputConnection API calls
         ic.beginBatchEdit()
-        ic.commitText("( ", 1)
-        ic.commitText(")", -1)
+        ic.commitText("ab", 1)
+        ic.commitText("c", -1)
         ic.endBatchEdit()
+
+        promise.value
         processChildEvents()
 
-        assertText("commit ()", ic, "( )")
+        assertText("committed text is \"abc\"", ic, "abc")
     }
 
     // Bug 1593683 - Cursor is jumping when using the arrow keys in input field on GBoard
@@ -1437,4 +1630,112 @@ class TextInputDelegateTest : BaseSessionTest() {
         finishComposingText(ic)
         assertText("commit foobaz1", ic, "foobaz1")
     }
+
+    @WithDisplay(width = 512, height = 512)
+    // Child process updates require having a display.
+    @Test
+    fun inputConnection_setComposingTextWithEmptyStringSpan() {
+        assumeThat("input only", id, equalTo("#input"))
+
+        setupContent("")
+        val ic = mainSession.textInput.onCreateInputConnection(EditorInfo())!!
+
+        var promise = mainSession.evaluatePromiseJS(
+            """
+            new Promise(r =>
+                document.querySelector('$id').addEventListener('input', () => {
+                    let input = document.querySelector('$id');
+                    let selStart = input.selectionStart;
+                    if (input.value.length == 4) {
+                      input.value = "123 4";
+                      selStart += 1;
+                      input.setSelectionRange(selStart, selStart);
+                      r();
+                    }
+                }));
+            """.trimIndent(),
+        )
+
+        pressKey(ic, KeyEvent.KEYCODE_1)
+        ic.setComposingText("", 1)
+        pressKey(ic, KeyEvent.KEYCODE_2)
+        ic.setComposingText("", 1)
+        pressKey(ic, KeyEvent.KEYCODE_3)
+        ic.setComposingText("", 1)
+
+        syncShadowText(ic)
+
+        assertSelectionAt("Update selection by key event", ic, 3)
+
+        pressKeyNoWait(ic, KeyEvent.KEYCODE_4)
+
+        // ATOK will set empty text that has a composing span.
+        val text = SpannableString("")
+        BaseInputConnection.setComposingSpans(text)
+        ic.setComposingText(text, 1)
+
+        promise.value
+
+        syncShadowText(ic)
+
+        assertSelectionAt("Update selection by setSelectionRange", ic, 5)
+    }
+
+    @Test
+    fun noHandleVolumeKeys() {
+        setupContent("")
+        mainSession.textInput.onCreateInputConnection(EditorInfo())
+
+        val keys = listOf(KeyEvent.KEYCODE_VOLUME_UP, KeyEvent.KEYCODE_VOLUME_DOWN, KeyEvent.KEYCODE_VOLUME_MUTE)
+        for (keyCode in keys) {
+            val time = SystemClock.uptimeMillis()
+            val keyEvent = KeyEvent(time, time, KeyEvent.ACTION_DOWN, keyCode, 0)
+            assertThat(
+                "We don't handle volume key on onKeyDown",
+                mainSession.textInput.onKeyDown(keyCode, keyEvent),
+                equalTo(false),
+            )
+            assertThat(
+                "We don't handle volume key on onKeyUp",
+                mainSession.textInput.onKeyUp(keyCode, KeyEvent.changeAction(keyEvent, KeyEvent.ACTION_UP)),
+                equalTo(false),
+            )
+        }
+    }
+
+    // Bug 1964660 - Sync selection without text change
+    @WithDisplay(width = 512, height = 512)
+    // Child process updates require having a display.
+    @Test
+    fun updateSelectionWithoutTextChange() {
+        assumeThat("input only", id, equalTo("#input"))
+
+        setupContent("[***]")
+        val ic = mainSession.textInput.onCreateInputConnection(EditorInfo())!!
+
+        mainSession.evaluateJS(
+            """
+            document.querySelector('$id').blur();
+
+            document.querySelector('$id').addEventListener('focus', () => {
+                document.querySelector('$id').setSelectionRange(1, 4);
+            });
+            document.querySelector('$id').addEventListener('input', () => {
+                document.querySelector('$id').value = '[***]';
+                document.querySelector('$id').setSelectionRange(1, 4);
+            });
+            """.trimIndent(),
+        )
+
+        mainSession.evaluateJS("document.querySelector('$id').focus()")
+        mainSession.waitUntilCalled(GeckoSession.TextInputDelegate::class, "restartInput")
+
+        processChildEvents()
+        assertSelection("selection isn't collapsed at start", ic, 1, 4)
+
+        pressKey(ic, KeyEvent.KEYCODE_A)
+        processChildEvents()
+        assertText("text isn't changed", ic, "[***]")
+        assertSelection("selection isn't collapsed", ic, 1, 4)
+    }
 }
