@@ -51,6 +51,8 @@ import android.view.ContextThemeWrapper;
 import android.view.Display;
 import android.view.InputDevice;
 import android.view.WindowManager;
+import android.view.inputmethod.InputMethodManager;
+import android.view.inputmethod.InputMethodSubtype;
 import android.webkit.MimeTypeMap;
 import androidx.annotation.Nullable;
 import androidx.collection.SimpleArrayMap;
@@ -64,14 +66,12 @@ import java.util.StringTokenizer;
 import org.jetbrains.annotations.NotNull;
 import org.mozilla.gecko.annotation.RobocopTarget;
 import org.mozilla.gecko.annotation.WrapForJNI;
-import org.mozilla.gecko.util.HardwareCodecCapabilityUtils;
 import org.mozilla.gecko.util.HardwareUtils;
 import org.mozilla.gecko.util.InputDeviceUtils;
 import org.mozilla.gecko.util.ProxySelector;
 import org.mozilla.gecko.util.ThreadUtils;
 import org.mozilla.geckoview.BuildConfig;
 import org.mozilla.geckoview.CrashHandler;
-import org.mozilla.geckoview.GeckoResult;
 import org.mozilla.geckoview.R;
 
 public class GeckoAppShell {
@@ -188,6 +188,13 @@ public class GeckoAppShell {
   // See also HardwareUtils.LOW_MEMORY_THRESHOLD_MB.
   private static final int HIGH_MEMORY_DEVICE_THRESHOLD_MB = 768;
 
+  /*
+   * Device RAM threshold requirement for adding additional headers.
+   * Keep in sync with RAM_THRESHOLD_MEGABYTES defined in
+   * https://searchfox.org/mozilla-central/rev/55944eaee1e358b5443eaedc8adcd37e3fd23fd3/mobile/android/fenix/app/src/main/java/org/mozilla/fenix/FenixApplication.kt#120
+   */
+  private static final int ADDITIONAL_SEARCH_HEADER_RAM_THRESHOLD_MEGABYTES = 1024;
+
   private static int sDensityDpi;
   private static Float sDensity;
   private static int sScreenDepth;
@@ -327,6 +334,7 @@ public class GeckoAppShell {
   private static synchronized boolean enableLocationUpdates(final boolean enable) {
     locationListeningRequested = enable;
     final boolean canListen = updateLocationListeners();
+    // canListen will be true even if paused. During paused, we keep requesting status.
     if (!canListen && locationListeningRequested) {
       // Didn't successfully start listener when requested
       locationListeningRequested = false;
@@ -344,11 +352,10 @@ public class GeckoAppShell {
     }
 
     if (!shouldListen) {
-      // Could not complete request, because paused
-      if (locationListeningRequested) {
-        return false;
+      if (!locationListeningRequested) {
+        // We are paused, so stop listening.
+        lm.removeUpdates(sAndroidListeners);
       }
-      lm.removeUpdates(sAndroidListeners);
       return true;
     }
 
@@ -616,7 +623,7 @@ public class GeckoAppShell {
         if (gGameRotationVectorSensor != null) {
           break;
         }
-        // Fallthrough
+      // Fallthrough
 
       case SENSOR_ROTATION_VECTOR:
         if (gRotationVectorSensor == null) {
@@ -629,7 +636,7 @@ public class GeckoAppShell {
         if (gRotationVectorSensor != null) {
           break;
         }
-        // Fallthrough
+      // Fallthrough
 
       case SENSOR_ORIENTATION:
         if (gOrientationSensor == null) {
@@ -697,14 +704,14 @@ public class GeckoAppShell {
           sm.unregisterListener(sAndroidListeners, gGameRotationVectorSensor);
           break;
         }
-        // Fallthrough
+      // Fallthrough
 
       case SENSOR_ROTATION_VECTOR:
         if (gRotationVectorSensor != null) {
           sm.unregisterListener(sAndroidListeners, gRotationVectorSensor);
           break;
         }
-        // Fallthrough
+      // Fallthrough
 
       case SENSOR_ORIENTATION:
         if (gOrientationSensor != null) {
@@ -742,17 +749,11 @@ public class GeckoAppShell {
 
   @WrapForJNI(calledFrom = "gecko")
   private static void moveTaskToBack() {
-    // This is a vestige, to be removed as full-screen support for GeckoView is implemented.
-  }
-
-  @WrapForJNI(calledFrom = "gecko")
-  private static boolean hasHWVP8Encoder() {
-    return HardwareCodecCapabilityUtils.hasHWVP8(true /* aIsEncoder */);
-  }
-
-  @WrapForJNI(calledFrom = "gecko")
-  private static boolean hasHWVP8Decoder() {
-    return HardwareCodecCapabilityUtils.hasHWVP8(false /* aIsEncoder */);
+    final Context applicationContext = getApplicationContext();
+    final Intent intent = new Intent(Intent.ACTION_MAIN);
+    intent.addCategory(Intent.CATEGORY_HOME);
+    intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
+    applicationContext.startActivity(intent);
   }
 
   @WrapForJNI(calledFrom = "gecko")
@@ -781,15 +782,16 @@ public class GeckoAppShell {
   }
 
   @WrapForJNI(dispatchTo = "gecko")
-  private static native void notifyAlertListener(String name, String topic, String cookie);
+  private static native void notifyAlertListener(
+      String name, String topic, String action, String origin);
 
   /**
    * Called by the NotificationListener to notify Gecko that a previously shown notification has
    * been closed.
    */
-  public static void onNotificationClose(final String name, final String cookie) {
+  public static void onNotificationClose(@NotNull final String name, @NotNull final String origin) {
     if (GeckoThread.isRunning()) {
-      notifyAlertListener(name, "alertfinished", cookie);
+      notifyAlertListener(name, "alertfinished", null, origin);
     }
   }
 
@@ -797,9 +799,10 @@ public class GeckoAppShell {
    * Called by the NotificationListener to notify Gecko that a previously shown notification has
    * been clicked on.
    */
-  public static void onNotificationClick(final String name, final String cookie) {
+  public static void onNotificationClick(
+      @NotNull final String name, @Nullable final String action, @NotNull final String origin) {
     if (GeckoThread.isRunning()) {
-      notifyAlertListener(name, "alertclickcallback", cookie);
+      notifyAlertListener(name, "alertclickcallback", action, origin);
     } else {
       GeckoThread.queueNativeCallUntil(
           GeckoThread.State.PROFILE_READY,
@@ -807,7 +810,9 @@ public class GeckoAppShell {
           "notifyAlertListener",
           name,
           "alertclickcallback",
-          cookie);
+          String.class,
+          action,
+          origin);
     }
   }
 
@@ -865,6 +870,12 @@ public class GeckoAppShell {
     return sTotalRam;
   }
 
+  @WrapForJNI(calledFrom = "gecko")
+  private static synchronized boolean isDeviceRamThresholdOkay() {
+    final Context applicationContext = getApplicationContext();
+    return getTotalRam(applicationContext) > ADDITIONAL_SEARCH_HEADER_RAM_THRESHOLD_MEGABYTES;
+  }
+
   private static boolean isHighMemoryDevice(final Context context) {
     return getTotalRam(context) > HIGH_MEMORY_DEVICE_THRESHOLD_MB;
   }
@@ -909,6 +920,33 @@ public class GeckoAppShell {
     return refreshRate;
   }
 
+  @WrapForJNI(calledFrom = "gecko")
+  private static boolean hasHDRScreen() {
+    if (Build.VERSION.SDK_INT < 24) {
+      return false;
+    }
+    final WindowManager wm =
+        (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
+    final Display display = wm.getDefaultDisplay();
+    if (Build.VERSION.SDK_INT >= 26) {
+      return display.isHdr();
+    }
+    final Display.HdrCapabilities hdrCapabilities = display.getHdrCapabilities();
+    if (hdrCapabilities == null) {
+      return false;
+    }
+    final int[] supportedHdrTypes = hdrCapabilities.getSupportedHdrTypes();
+    for (final int type : supportedHdrTypes) {
+      if (type == Display.HdrCapabilities.HDR_TYPE_HDR10
+          || type == Display.HdrCapabilities.HDR_TYPE_HDR10_PLUS
+          || type == Display.HdrCapabilities.HDR_TYPE_HLG
+          || type == Display.HdrCapabilities.HDR_TYPE_DOLBY_VISION) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   @WrapForJNI(calledFrom = "gecko")
   private static void performHapticFeedback(final boolean aIsLongPress) {
     // Don't perform haptic feedback if a vibration is currently playing,
@@ -1112,6 +1150,25 @@ public class GeckoAppShell {
     return result;
   }
 
+  @WrapForJNI(calledFrom = "gecko")
+  private static String getKeyboardLayout() {
+    final InputMethodManager imm =
+        (InputMethodManager) getApplicationContext().getSystemService(Context.INPUT_METHOD_SERVICE);
+    final InputMethodSubtype ims = imm.getCurrentInputMethodSubtype();
+    if (ims == null) {
+      return null;
+    }
+
+    // TODO(m_kato):
+    // Android 16 will have `layout related APIs such as setLayoutLabelNonLocalized
+    // to get keyboard layout label.
+    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
+      return ims.getLanguageTag();
+    } else {
+      return ims.getLocale();
+    }
+  }
+
   @WrapForJNI(calledFrom = "gecko")
   private static byte[] getIconForExtension(final String aExt, final int iconSize) {
     try {
@@ -1387,6 +1444,58 @@ public class GeckoAppShell {
     return result;
   }
 
+  /*
+   * Keep in sync with PointingDevices in LookAndFeel.h
+   */
+  private static final int POINTING_DEVICE_NONE = 0x00000000;
+  private static final int POINTING_DEVICE_MOUSE = 0x00000001;
+  private static final int POINTING_DEVICE_TOUCH = 0x00000002;
+  private static final int POINTING_DEVICE_PEN = 0x00000004;
+
+  private static int getPointingDeviceKinds(final InputDevice inputDevice) {
+    int result = POINTING_DEVICE_NONE;
+    final int sources = inputDevice.getSources();
+
+    // TODO(krosylight): For now this code is for telemetry purpose, but ultimately we want to
+    // replace the capabilities code above and move the capabilities computation into layout. We'll
+    // then have to add all the extra devices too that are not mouse/touch/pen. (Bug 1918207)
+    // We don't treat other devices properly for pointerType after all:
+    // https://searchfox.org/mozilla-central/rev/3b59c739df66574d94022a684596845cd05e7c65/mobile/android/geckoview/src/main/java/org/mozilla/geckoview/PanZoomController.java#749-761
+
+    if (hasInputDeviceSource(sources, InputDevice.SOURCE_MOUSE)) {
+      result |= POINTING_DEVICE_MOUSE;
+    }
+    if (hasInputDeviceSource(sources, InputDevice.SOURCE_TOUCHSCREEN)) {
+      result |= POINTING_DEVICE_TOUCH;
+    }
+    if (hasInputDeviceSource(sources, InputDevice.SOURCE_STYLUS)) {
+      result |= POINTING_DEVICE_PEN;
+    }
+    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M
+        && hasInputDeviceSource(sources, InputDevice.SOURCE_BLUETOOTH_STYLUS)) {
+      result |= POINTING_DEVICE_PEN;
+    }
+
+    return result;
+  }
+
+  @WrapForJNI(calledFrom = "gecko")
+  // For pointing devices telemetry.
+  private static int getPointingDeviceKinds() {
+    int result = POINTING_DEVICE_NONE;
+
+    for (final int deviceId : InputDevice.getDeviceIds()) {
+      final InputDevice inputDevice = InputDevice.getDevice(deviceId);
+      if (inputDevice == null || !InputDeviceUtils.isPointerTypeDevice(inputDevice)) {
+        continue;
+      }
+
+      result |= getPointingDeviceKinds(inputDevice);
+    }
+
+    return result;
+  }
+
   private static boolean hasInputDeviceSource(final int sources, final int inputDeviceSource) {
     return (sources & inputDeviceSource) == inputDeviceSource;
   }
@@ -1471,6 +1580,11 @@ public class GeckoAppShell {
 
   @WrapForJNI(calledFrom = "any")
   public static int getAudioOutputFramesPerBuffer() {
+    if (BuildConfig.DEBUG_BUILD && isIsolatedProcess()) {
+      // AudioManager.getProperty won't return on isolated process
+      throw new UnsupportedOperationException(
+          "getAudioOutputFramesPerBuffer is not supported in isolated processes");
+    }
     final int DEFAULT = 512;
 
     final AudioManager am =
@@ -1487,6 +1601,11 @@ public class GeckoAppShell {
 
   @WrapForJNI(calledFrom = "any")
   public static int getAudioOutputSampleRate() {
+    if (BuildConfig.DEBUG_BUILD && isIsolatedProcess()) {
+      // AudioManager.getProperty won't return on isolated process
+      throw new UnsupportedOperationException(
+          "getAudioOutputSampleRate is not supported in isolated processes");
+    }
     final int DEFAULT = 44100;
 
     final AudioManager am =
@@ -1583,17 +1702,23 @@ public class GeckoAppShell {
   @WrapForJNI
   public static native boolean isParentProcess();
 
-  /**
-   * Returns a GeckoResult that will be completed to true if the GPU process is enabled and false if
-   * it is disabled.
-   */
   @WrapForJNI
-  public static native GeckoResult<Boolean> isGpuProcessEnabled();
+  public static native boolean isGpuProcessEnabled();
 
+  @WrapForJNI
+  public static native boolean isInteractiveWidgetDefaultResizesVisual();
+
+  @WrapForJNI
   @SuppressLint("NewApi")
   public static boolean isIsolatedProcess() {
     // This method was added in SDK 16 but remained hidden until SDK 28, meaning we are okay to call
     // this on any SDK level but must suppress the new API lint.
     return android.os.Process.isIsolated();
   }
+
+  @WrapForJNI(dispatchTo = "gecko")
+  public static native void onSystemLocaleChanged();
+
+  @WrapForJNI(dispatchTo = "gecko")
+  public static native void onTimezoneChanged();
 }
