@@ -192,6 +192,7 @@ import org.mozilla.geckoview.SessionTextInput.EditableListener.IMEState;
         unicodeChar >= ' '
             ? unicodeChar
             : unmodifiedMetaState != metaState ? unmodifiedUnicodeChar : 0;
+    final boolean waitingReply = GeckoInputConnection.isMediaKeyEvent(event);
 
     // If a modifier (e.g. meta key) caused a different character to be entered, we
     // drop that modifier from the metastate for the generated keypress event.
@@ -219,6 +220,7 @@ import org.mozilla.geckoview.SessionTextInput.EditableListener.IMEState;
         event.getRepeatCount(),
         event.getFlags(),
         isSynthesizedImeKey,
+        waitingReply,
         event);
   }
 
@@ -1897,7 +1899,7 @@ import org.mozilla.geckoview.SessionTextInput.EditableListener.IMEState;
       if (modeHint.equals("tel")) {
         outAttrs.inputType = InputType.TYPE_CLASS_PHONE;
       } else if (modeHint.equals("url")) {
-        outAttrs.inputType = InputType.TYPE_TEXT_VARIATION_URI;
+        outAttrs.inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_URI;
       } else if (modeHint.equals("email")) {
         outAttrs.inputType |= InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS;
       } else if (modeHint.equals("numeric")) {
@@ -2091,6 +2093,12 @@ import org.mozilla.geckoview.SessionTextInput.EditableListener.IMEState;
       return;
     }
 
+    if (start != end && !causedOnlyByComposition) {
+      // selection isn't collapsed. This change is caused by web content side.
+      // So we have to update the selection in Java side.
+      mIgnoreSelectionChange = false;
+    }
+
     if (mIgnoreSelectionChange) {
       mIgnoreSelectionChange = false;
     } else {
@@ -2642,7 +2650,7 @@ import org.mozilla.geckoview.SessionTextInput.EditableListener.IMEState;
       case KeyEvent.KEYCODE_VOLUME_DOWN:
       case KeyEvent.KEYCODE_VOLUME_MUTE:
       case KeyEvent.KEYCODE_SEARCH:
-        // ignore HEADSETHOOK to allow hold-for-voice-search to work
+      // ignore HEADSETHOOK to allow hold-for-voice-search to work
       case KeyEvent.KEYCODE_HEADSETHOOK:
         return false;
     }
