@@ -70,6 +70,7 @@ import org.mozilla.gecko.util.ThreadUtils;
   private ExtractedTextRequest mUpdateRequest;
   private final InputConnection mKeyInputConnection;
   private CursorAnchorInfo.Builder mCursorAnchorInfoBuilder;
+  private boolean mIsPrivateBrowsing;
 
   public static SessionTextInput.InputConnectionClient create(
       final GeckoSession session,
@@ -208,12 +209,13 @@ import org.mozilla.gecko.util.ThreadUtils;
         // If selection is empty, we'll select everything
         if (selStart == selEnd) {
           // Fill the clipboard
-          Clipboard.setText(view.getContext(), editable);
+          Clipboard.setText(view.getContext(), editable, mIsPrivateBrowsing);
           editable.clear();
         } else {
           Clipboard.setText(
               view.getContext(),
-              editable.subSequence(Math.min(selStart, selEnd), Math.max(selStart, selEnd)));
+              editable.subSequence(Math.min(selStart, selEnd), Math.max(selStart, selEnd)),
+              mIsPrivateBrowsing);
           editable.delete(selStart, selEnd);
         }
         break;
@@ -231,7 +233,7 @@ import org.mozilla.gecko.util.ThreadUtils;
                 : editable
                     .toString()
                     .substring(Math.min(selStart, selEnd), Math.max(selStart, selEnd));
-        Clipboard.setText(view.getContext(), copiedText);
+        Clipboard.setText(view.getContext(), copiedText, mIsPrivateBrowsing);
         break;
     }
     return true;
@@ -474,6 +476,9 @@ import org.mozilla.gecko.util.ThreadUtils;
 
   @Override // SessionTextInput.EditableListener
   public void onDefaultKeyEvent(final KeyEvent event) {
+    if (DEBUG) {
+      Log.d(LOGTAG, "onDefaultKeyEvent: " + event);
+    }
     ThreadUtils.runOnUiThread(
         new Runnable() {
           @Override
@@ -603,6 +608,9 @@ import org.mozilla.gecko.util.ThreadUtils;
       outAttrs.imeOptions |= EditorInfo.IME_FLAG_NO_EXTRACT_UI | EditorInfo.IME_FLAG_NO_FULLSCREEN;
     }
 
+    mIsPrivateBrowsing =
+        ((outAttrs.imeOptions & InputMethods.IME_FLAG_NO_PERSONALIZED_LEARNING) != 0);
+
     if (DEBUG) {
       Log.d(
           LOGTAG,
@@ -640,6 +648,34 @@ import org.mozilla.gecko.util.ThreadUtils;
     return true;
   }
 
+  @Override
+  public boolean setComposingText(final CharSequence text, final int newCursorPosition) {
+    final Editable content = getEditable();
+    if (content != null && text.length() == 0 && newCursorPosition == 1) {
+      final int composingStart = getComposingSpanStart(content);
+      final int composingEnd = getComposingSpanEnd(content);
+      if (composingStart < 0 || composingEnd < 0) {
+        final int selStart = Selection.getSelectionStart(content);
+        final int selEnd = Selection.getSelectionEnd(content);
+        if (selStart == selEnd) {
+          // We have no composition and trying composing text is also empty.
+          // If newCursorPosition is 1 and inserted text is empty, it sets the selection to
+          // inserted position.
+          // So if current selection is collapsed, this does nothing.
+          if (DEBUG) {
+            Log.d(
+                LOGTAG,
+                "setComposingText does nothing. Becasue, although we have no composing text and IME"
+                    + " tries to set empty string.");
+          }
+          return true;
+        }
+      }
+    }
+
+    return super.setComposingText(text, newCursorPosition);
+  }
+
   @Override
   public boolean commitText(final CharSequence text, final int newCursorPosition) {
     if (InputMethods.shouldCommitCharAsKey(mCurrentInputMethod)
@@ -742,8 +778,7 @@ import org.mozilla.gecko.util.ThreadUtils;
     return event;
   }
 
-  // Called by OnDefaultKeyEvent handler, up from Gecko
-  /* package */ void performDefaultKeyAction(final KeyEvent event) {
+  /* package */ static boolean isMediaKeyEvent(final KeyEvent event) {
     switch (event.getKeyCode()) {
       case KeyEvent.KEYCODE_MUTE:
       case KeyEvent.KEYCODE_HEADSETHOOK:
@@ -758,18 +793,27 @@ import org.mozilla.gecko.util.ThreadUtils;
       case KeyEvent.KEYCODE_MEDIA_FAST_FORWARD:
       case KeyEvent.KEYCODE_MEDIA_CLOSE:
       case KeyEvent.KEYCODE_MEDIA_EJECT:
-      case KeyEvent.KEYCODE_MEDIA_AUDIO_TRACK:
-        // Forward media keypresses to the registered handler so headset controls work
-        // Does the same thing as Chromium
-        // https://chromium.googlesource.com/chromium/src/+/49.0.2623.67/chrome/android/java/src/org/chromium/chrome/browser/tab/TabWebContentsDelegateAndroid.java#445
-        // These are all the keys dispatchMediaKeyEvent supports.
-        final Context viewContext = getView().getContext();
-        final AudioManager am = (AudioManager) viewContext.getSystemService(Context.AUDIO_SERVICE);
-        am.dispatchMediaKeyEvent(event);
-        break;
+        return true;
+      default:
+        return false;
     }
   }
 
+  // Called by OnDefaultKeyEvent handler, up from Gecko
+  /* package */ void performDefaultKeyAction(final KeyEvent event) {
+    if (!isMediaKeyEvent(event)) {
+      return;
+    }
+
+    // Forward media keypresses to the registered handler so headset controls work
+    // Does the same thing as Chromium
+    // https://chromium.googlesource.com/chromium/src/+/49.0.2623.67/chrome/android/java/src/org/chromium/chrome/browser/tab/TabWebContentsDelegateAndroid.java#445
+    // These are all the keys dispatchMediaKeyEvent supports.
+    final Context viewContext = getView().getContext();
+    final AudioManager am = (AudioManager) viewContext.getSystemService(Context.AUDIO_SERVICE);
+    am.dispatchMediaKeyEvent(event);
+  }
+
   @TargetApi(Build.VERSION_CODES.N_MR1)
   @Override
   public boolean commitContent(
