@@ -368,6 +368,18 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
       return this;
     }
 
+    /**
+     * Set whether the user wants to disable the CrashPullController.Delegate from showing the
+     * notification requesting crash pull.
+     *
+     * @param enabled A flag determining whether the notification should be shown.
+     * @return The Builder instance.
+     */
+    public @NonNull Builder crashPullNeverShowAgain(final boolean enabled) {
+      getSettings().mRemoteSettingCrashPullNeverShowAgain.set(enabled);
+      return this;
+    }
+
     /**
      * Sets whether Session History in Parent (SHIP) should be disabled or not.
      *
@@ -555,6 +567,17 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
       return this;
     }
 
+    /**
+     * Sets whether or not local network access (LNA) blocking is enabled
+     *
+     * @param enabled flag indicating whether or not local network access (LNA) blocking is enabled
+     * @return The builder instance
+     */
+    public @NonNull Builder setLnaBlockingEnabled(@NonNull final Boolean enabled) {
+      getSettings().setLnaBlockingEnabled(enabled);
+      return this;
+    }
+
     /**
      * Sets whether and how DNS-over-HTTPS (Trusted Recursive Resolver) is configured.
      *
@@ -613,6 +636,33 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
       getSettings().mLowMemoryDetection = enable;
       return this;
     }
+
+    /**
+     * Sets whether the same document navigation should override the load type or not.
+     *
+     * @param value A flag determining whether same document navigation should override the load
+     *     type or not.
+     * @return The builder instance.
+     */
+    public @NonNull Builder setSameDocumentNavigationOverridesLoadType(final boolean value) {
+      getSettings().setSameDocumentNavigationOverridesLoadType(value);
+      return this;
+    }
+
+    /**
+     * Sets the uri to force-disable the same document navigation overriding the load type. If it is
+     * an empty string (default value), there's no specific domain that the same document navigation
+     * overriding the load type is disabled.
+     *
+     * @param uri URI that will be used to force-disable the same document navigation overriding the
+     *     load type on a specific domain.
+     * @return The builder instance.
+     */
+    public @NonNull Builder setSameDocumentNavigationOverridesLoadTypeForceDisable(
+        @NonNull final String uri) {
+      getSettings().setSameDocumentNavigationOverridesLoadTypeForceDisable(uri);
+      return this;
+    }
   }
 
   private GeckoRuntime mRuntime;
@@ -665,6 +715,8 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
       new Pref<Boolean>("dom.security.https_only_mode", false);
   /* package */ final Pref<Boolean> mHttpsOnlyPrivateMode =
       new Pref<Boolean>("dom.security.https_only_mode_pbm", false);
+
+  /* package */ final Pref<Boolean> mLnaBlockingEnabled = new Pref<>("network.lna.blocking", false);
   /* package */ final PrefWithoutDefault<Integer> mTrustedRecursiveResolverMode =
       new PrefWithoutDefault<>("network.trr.mode");
   /* package */ final PrefWithoutDefault<String> mTrustedRecursiveResolverUri =
@@ -700,6 +752,10 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
       new Pref<Boolean>("javascript.options.use_fdlibm_for_sin_cos_tan", false);
   /* package */ final Pref<Integer> mUserCharacteristicPingCurrentVersion =
       new Pref<>("toolkit.telemetry.user_characteristics_ping.current_version", 0);
+  /* package */ final PrefWithoutDefault<Boolean> mBaselineFingerprintingProtection =
+      new PrefWithoutDefault<>("privacy.baselineFingerprintingProtection");
+  /* package */ final PrefWithoutDefault<String> mBaselineFingerprintingProtectionOverrides =
+      new PrefWithoutDefault<>("privacy.baselineFingerprintingProtection.overrides");
   /* package */ PrefWithoutDefault<Boolean> mDisableShip =
       new PrefWithoutDefault<Boolean>("fission.disableSessionHistoryInParent");
   /* package */ final Pref<Boolean> mFetchPriorityEnabled =
@@ -712,10 +768,21 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
       new Pref<Boolean>("network.cookie.cookieBehavior.optInPartitioning.pbmode", false);
   /* package */ final Pref<Integer> mCertificateTransparencyMode =
       new Pref<Integer>("security.pki.certificate_transparency.mode", 0);
-  /* package */ final Pref<Boolean> mPostQuantumKeyExchangeTLSEnabled =
-      new Pref<Boolean>("security.tls.enable_kyber", false);
-  /* package */ final Pref<Boolean> mPostQuantumKeyExchangeHttp3Enabled =
-      new Pref<Boolean>("network.http.http3.enable_kyber", false);
+  /* package */ final PrefWithoutDefault<Boolean> mPostQuantumKeyExchangeTLSEnabled =
+      new PrefWithoutDefault<Boolean>("security.tls.enable_kyber");
+  /* package */ final PrefWithoutDefault<Boolean> mPostQuantumKeyExchangeHttp3Enabled =
+      new PrefWithoutDefault<Boolean>("network.http.http3.enable_kyber");
+  /* package */ final Pref<Boolean> mDohAutoselectEnabled =
+      new Pref<Boolean>("network.android_doh.autoselect_enabled", false);
+  /* package */ final Pref<Boolean> mSameDocumentNavigationOverridesLoadType =
+      new Pref<Boolean>("docshell.shistory.sameDocumentNavigationOverridesLoadType", true);
+  /* package */ final Pref<String> mSameDocumentNavigationOverridesLoadTypeForceDisable =
+      new Pref<String>(
+          "docshell.shistory.sameDocumentNavigationOverridesLoadType.forceDisable", "");
+  /* package */ final Pref<String> mBannedPorts =
+      new Pref<String>("network.security.ports.banned", "");
+  /* package */ final PrefWithoutDefault<Boolean> mRemoteSettingCrashPullNeverShowAgain =
+      new PrefWithoutDefault<Boolean>("browser.crashReports.requestedNeverShowAgain");
 
   /* package */ int mPreferredColorScheme = COLOR_SCHEME_SYSTEM;
 
@@ -884,7 +951,9 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
    * Set the Fingerprint protection overrides
    *
    * @param overrides The overrides value to add or remove fingerprinting protection targets. Please
-   *     check RFPTargets.inc for all supported targets.
+   *     check RFPTargets.inc for all supported targets. The format of the overrides is
+   *     +Target1,-Target2,+Target3 where + means add and - means remove. The targets are separated
+   *     by commas. The default value is empty string.
    * @return This GeckoRuntimeSettings instance
    */
   public @NonNull GeckoRuntimeSettings setFingerprintingProtectionOverrides(
@@ -941,6 +1010,50 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     return mFdlibmMathEnabled.get();
   }
 
+  /**
+   * Set the Baseline fingerprint protection.
+   *
+   * @param enabled Whether we set the pref to true or false
+   * @return This GeckoRuntimeSettings instance
+   */
+  public @NonNull GeckoRuntimeSettings setBaselineFingerprintingProtection(final boolean enabled) {
+    mBaselineFingerprintingProtection.commit(enabled);
+    return this;
+  }
+
+  /**
+   * Set the Baseline fingerprint protection overrides
+   *
+   * @param overrides The overrides value to add or remove fingerprinting protection targets. Please
+   *     check RFPTargets.inc for all supported targets. The format of the overrides is
+   *     +Target1,-Target2,+Target3 where + means add and - means remove. The targets are separated
+   *     by commas. The default value is empty string.
+   * @return This GeckoRuntimeSettings instance
+   */
+  public @NonNull GeckoRuntimeSettings setBaselineFingerprintingProtectionOverrides(
+      @NonNull final String overrides) {
+    mBaselineFingerprintingProtectionOverrides.commit(overrides);
+    return this;
+  }
+
+  /**
+   * Get whether Fingerprint protection is enabled private browsing mode.
+   *
+   * @return Whether Fingerprint protection is enabled private browsing mode.
+   */
+  public @Nullable Boolean getBaselineFingerprintingProtection() {
+    return mBaselineFingerprintingProtection.get();
+  }
+
+  /**
+   * Get Fingerprint protection overrides.
+   *
+   * @return The string of the fingerprinting protection overrides.
+   */
+  public @Nullable String getBaselineFingerprintingProtectionOverrides() {
+    return mBaselineFingerprintingProtectionOverrides.get();
+  }
+
   /**
    * Set the pref to control the cookie behavior opt-in partitioning.
    *
@@ -964,6 +1077,19 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     return this;
   }
 
+  /**
+   * Set the pref to control whether the CrashPullController.Delegate may show the crash pull
+   * notification.
+   *
+   * @param enabled Whether we set the pref to true or false
+   * @return This GeckoRuntimeSettings instance
+   */
+  public @NonNull GeckoRuntimeSettings setRemoteSettingCrashPullNeverShowAgain(
+      final boolean enabled) {
+    mRemoteSettingCrashPullNeverShowAgain.commit(enabled);
+    return this;
+  }
+
   /**
    * Get whether the cookie behavior opt-in partitioning is enabled.
    *
@@ -1586,6 +1712,35 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     return this;
   }
 
+  /**
+   * Set the pref to control whether network.android_doh.autoselect_enabled is enabled. When the
+   * pref is enabled the browser will automatically select a DoH provider from a region specific
+   * list provided by remote-settings if the browser is using the 'Default protection' mode for DNS
+   * over HTTPS. This means `network.trr.mode` was previously set to 0 by calling
+   * `setTrustedRecursiveResolverMode(TRR_MODE_OFF)`
+   *
+   * <p>When automatic selection is enabled the browser will run heuristic checks to determine
+   * whether DoH can be enabled for the current network (checks for parental controls, split horizon
+   * DNS, canary domain, etc) and if these pass it will set the `doh-rollout.mode` pref to 2
+   * (TRR_MODE_FIRST) and `doh-rollout.uri` to the URL of the automatically selected provider.
+   *
+   * @param enabled Whether to enable the DoHController component
+   * @return This GeckoRuntimeSettings instance
+   */
+  public @NonNull GeckoRuntimeSettings setDohAutoselectEnabled(final boolean enabled) {
+    mDohAutoselectEnabled.commit(enabled);
+    return this;
+  }
+
+  /**
+   * Get whether network.trr.android_rollout_enabled is enabled.
+   *
+   * @return Whether the DoHController component will be initialized
+   */
+  public boolean getDohAutoselectEnabled() {
+    return mDohAutoselectEnabled.get();
+  }
+
   /**
    * Gets whether double-tap zooming is enabled.
    *
@@ -1813,6 +1968,26 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     return ALLOW_ALL;
   }
 
+  /**
+   * Sets whether or not local network access (LNA) blocking is enabled
+   *
+   * @param enabled flag indicating whether or not local network access blocking is enabled
+   * @return The updated instance of {@link GeckoRuntimeSettings}
+   */
+  public @NonNull GeckoRuntimeSettings setLnaBlockingEnabled(final boolean enabled) {
+    mLnaBlockingEnabled.commit(enabled);
+    return this;
+  }
+
+  /**
+   * Gets whether or not local network access (LNA) blocking is enabled
+   *
+   * @return Boolean indicating whether LNA blocking is enabled or not.
+   */
+  public boolean getLnaBlockingEnabled() {
+    return mLnaBlockingEnabled.get();
+  }
+
   /**
    * Set whether and where insecure (non-HTTPS) connections are allowed.
    *
@@ -2061,7 +2236,29 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
    * @return Whether post-quantum key exchange mechanisms are enabled.
    */
   public @NonNull boolean getPostQuantumKeyExchangeEnabled() {
-    return mPostQuantumKeyExchangeTLSEnabled.get() && mPostQuantumKeyExchangeHttp3Enabled.get();
+    final Boolean tlsEnabled = mPostQuantumKeyExchangeTLSEnabled.get();
+    final Boolean h3Enabled = mPostQuantumKeyExchangeHttp3Enabled.get();
+    return (tlsEnabled != null && tlsEnabled) && (h3Enabled != null && h3Enabled);
+  }
+
+  /**
+   * Set the preference that controls which destination ports Firefox should refuse to connect to.
+   *
+   * @param portList Comma separated list of ports
+   * @return This GeckoRuntimeSettings instance
+   */
+  public @NonNull GeckoRuntimeSettings setBannedPorts(final @NonNull String portList) {
+    mBannedPorts.commit(portList);
+    return this;
+  }
+
+  /**
+   * Get the list of banned ports as a comma separated string
+   *
+   * @return a String containing the list of banned ports
+   */
+  public @NonNull String getBannedPorts() {
+    return mBannedPorts.get();
   }
 
   // For internal use only
@@ -2071,6 +2268,55 @@ public final class GeckoRuntimeSettings extends RuntimeSettings {
     return this;
   }
 
+  /**
+   * Sets whether the same document navigation should override the load type or not.
+   *
+   * @param value A flag determining whether same document navigation should override the load type
+   *     or not.
+   * @return This GeckoRuntimeSettings instance.
+   */
+  public @NonNull GeckoRuntimeSettings setSameDocumentNavigationOverridesLoadType(
+      final boolean value) {
+    mSameDocumentNavigationOverridesLoadType.commit(value);
+    return this;
+  }
+
+  /**
+   * Gets whether the same document navigation should override the load type or not.
+   *
+   * @return Whether the same document navigation should override the load type or not.
+   */
+  public @NonNull boolean getSameDocumentNavigationOverridesLoadType() {
+    return mSameDocumentNavigationOverridesLoadType.get();
+  }
+
+  /**
+   * Sets the uri to force-disable the same document navigation overriding the load type. If it is
+   * an empty string (default value), there's no specific domain that the same document navigation
+   * overriding the load type is disabled.
+   *
+   * @param uri URI that will be used to force-disable the same document navigation overriding the
+   *     load type on a specific domain.
+   * @return This GeckoRuntimeSettings instance.
+   */
+  public @NonNull GeckoRuntimeSettings setSameDocumentNavigationOverridesLoadTypeForceDisable(
+      @NonNull final String uri) {
+    mSameDocumentNavigationOverridesLoadTypeForceDisable.commit(uri);
+    return this;
+  }
+
+  /**
+   * Gets the uri to force-disable the same document navigation overriding the load type. If it is
+   * an empty string (default value), there's no specific domain that the same document navigation
+   * overriding the load type is disabled.
+   *
+   * @return URI that will be used to force-disable the same document navigation overriding the load
+   *     type on a specific domain.
+   */
+  public @NonNull String getSameDocumentNavigationOverridesLoadTypeForceDisable() {
+    return mSameDocumentNavigationOverridesLoadTypeForceDisable.get();
+  }
+
   @Override // Parcelable
   public void writeToParcel(final Parcel out, final int flags) {
     super.writeToParcel(out, flags);
