@@ -247,6 +247,7 @@ public class WebExtensionController {
    */
   @UiThread
   public interface PromptDelegate {
+
     /**
      * Called whenever a new extension is being installed. This is intended as an opportunity for
      * the app to prompt the user for the permissions required by this extension.
@@ -254,13 +255,19 @@ public class WebExtensionController {
      * @param extension The {@link WebExtension} that is about to be installed. You can use {@link
      *     WebExtension#metaData} to gather information about this extension when building the user
      *     prompt dialog.
-     * @return A {@link GeckoResult} that completes to either {@link AllowOrDeny#ALLOW ALLOW} if
-     *     this extension should be installed or {@link AllowOrDeny#DENY DENY} if this extension
-     *     should not be installed. A null value will be interpreted as {@link AllowOrDeny#DENY
-     *     DENY}.
+     * @param permissions The list of permissions that are granted during installation.
+     * @param origins The list of origins that are granted during installation.
+     * @param dataCollectionPermissions The list of data collection permissions that are requested
+     *     during installation.
+     * @return A {@link GeckoResult} that completes with a {@link
+     *     WebExtension.PermissionPromptResponse} containing all the details from the user response.
      */
     @Nullable
-    default GeckoResult<AllowOrDeny> onInstallPrompt(final @NonNull WebExtension extension) {
+    default GeckoResult<WebExtension.PermissionPromptResponse> onInstallPromptRequest(
+        @NonNull final WebExtension extension,
+        @NonNull final String[] permissions,
+        @NonNull final String[] origins,
+        @NonNull final String[] dataCollectionPermissions) {
       return null;
     }
 
@@ -268,6 +275,8 @@ public class WebExtensionController {
      * Called whenever an updated extension has new permissions. This is intended as an opportunity
      * for the app to prompt the user for the new permissions required by this extension.
      *
+     * @deprecated Use onUpdatePrompt(WebExtension, String[], String[], String[]) to account for
+     *     data collection permissions.
      * @param currentlyInstalled The {@link WebExtension} that is currently installed.
      * @param updatedExtension The {@link WebExtension} that will replace the previous extension.
      * @param newPermissions The new permissions that are needed.
@@ -277,6 +286,8 @@ public class WebExtensionController {
      *     not be update. A null value will be interpreted as {@link AllowOrDeny#DENY DENY}.
      */
     @Nullable
+    @Deprecated
+    @DeprecationSchedule(id = "web-extension-on-update-prompt", version = 144)
     default GeckoResult<AllowOrDeny> onUpdatePrompt(
         @NonNull final WebExtension currentlyInstalled,
         @NonNull final WebExtension updatedExtension,
@@ -285,6 +296,52 @@ public class WebExtensionController {
       return null;
     }
 
+    /**
+     * Called whenever an updated extension has new permissions. This is intended as an opportunity
+     * for the app to prompt the user for the new permissions required by this extension.
+     *
+     * @param extension The {@link WebExtension} being updated.
+     * @param newPermissions The new permissions that are needed.
+     * @param newOrigins The new origins that are needed.
+     * @param newDataCollectionPermissions The new data collection permissions that are needed.
+     * @return A {@link GeckoResult} that completes to either {@link AllowOrDeny#ALLOW ALLOW} if
+     *     this extension should be update or {@link AllowOrDeny#DENY DENY} if this extension should
+     *     not be update. A null value will be interpreted as {@link AllowOrDeny#DENY DENY}.
+     */
+    @Nullable
+    default GeckoResult<AllowOrDeny> onUpdatePrompt(
+        @NonNull final WebExtension extension,
+        @NonNull final String[] newPermissions,
+        @NonNull final String[] newOrigins,
+        @NonNull final String[] newDataCollectionPermissions) {
+      return null;
+    }
+
+    /**
+     * Called whenever permissions are requested. This is intended as an opportunity for the app to
+     * prompt the user for the permissions required by this extension at runtime.
+     *
+     * @deprecated Use onOptionalPrompt(WebExtension, String[], String[], String[]) to account for
+     *     data collection permissions.
+     * @param extension The {@link WebExtension} that is about to be installed. You can use {@link
+     *     WebExtension#metaData} to gather information about this extension when building the user
+     *     prompt dialog.
+     * @param permissions The permissions that are requested.
+     * @param origins The requested host permissions.
+     * @return A {@link GeckoResult} that completes to either {@link AllowOrDeny#ALLOW ALLOW} if the
+     *     request should be approved or {@link AllowOrDeny#DENY DENY} if the request should be
+     *     denied. A null value will be interpreted as {@link AllowOrDeny#DENY DENY}.
+     */
+    @Nullable
+    @Deprecated
+    @DeprecationSchedule(id = "web-extension-on-optional-prompt", version = 144)
+    default GeckoResult<AllowOrDeny> onOptionalPrompt(
+        final @NonNull WebExtension extension,
+        final @NonNull String[] permissions,
+        final @NonNull String[] origins) {
+      return null;
+    }
+
     /**
      * Called whenever permissions are requested. This is intended as an opportunity for the app to
      * prompt the user for the permissions required by this extension at runtime.
@@ -294,6 +351,7 @@ public class WebExtensionController {
      *     prompt dialog.
      * @param permissions The permissions that are requested.
      * @param origins The requested host permissions.
+     * @param dataCollectionPermissions The requested data collection permissions.
      * @return A {@link GeckoResult} that completes to either {@link AllowOrDeny#ALLOW ALLOW} if the
      *     request should be approved or {@link AllowOrDeny#DENY DENY} if the request should be
      *     denied. A null value will be interpreted as {@link AllowOrDeny#DENY DENY}.
@@ -302,7 +360,8 @@ public class WebExtensionController {
     default GeckoResult<AllowOrDeny> onOptionalPrompt(
         final @NonNull WebExtension extension,
         final @NonNull String[] permissions,
-        final @NonNull String[] origins) {
+        final @NonNull String[] origins,
+        final @NonNull String[] dataCollectionPermissions) {
       return null;
     }
   }
@@ -609,7 +668,8 @@ public class WebExtensionController {
    *
    * <p>When calling this method, the GeckoView library will download the extension, validate its
    * manifest and signature, and give you an opportunity to verify its permissions through {@link
-   * PromptDelegate#installPrompt}, you can use this method to prompt the user if appropriate.
+   * PromptDelegate#onInstallPromptRequest}, you can use this method to prompt the user if
+   * appropriate.
    *
    * @param uri URI to the extension's <code>.xpi</code> package. This can be a remote <code>https:
    *     </code> URI or a local <code>file:</code> or <code>resource:</code> URI. Note: the app
@@ -623,7 +683,7 @@ public class WebExtensionController {
    *     exceptionally with a {@link WebExtension.InstallException InstallException} that will
    *     contain the relevant error code in {@link WebExtension.InstallException#code
    *     InstallException#code}.
-   * @see PromptDelegate#installPrompt
+   * @see PromptDelegate#onInstallPromptRequest(WebExtension, String[], String[], String[])
    * @see WebExtension.InstallException.ErrorCodes
    * @see WebExtension#metaData
    */
@@ -632,7 +692,7 @@ public class WebExtensionController {
   public GeckoResult<WebExtension> install(
       final @NonNull String uri, final @Nullable @InstallationMethod String installationMethod) {
     final InstallCanceller canceller = new InstallCanceller();
-    final GeckoBundle bundle = new GeckoBundle(2);
+    final GeckoBundle bundle = new GeckoBundle(3);
     bundle.putString("locationUri", uri);
     bundle.putString("installId", canceller.installId);
     bundle.putString("installMethod", installationMethod);
@@ -660,9 +720,9 @@ public class WebExtensionController {
    *
    * <p>When calling this method, the GeckoView library will download the extension, validate its
    * manifest and signature, and give you an opportunity to verify its permissions through {@link
-   * PromptDelegate#installPrompt}, you can use this method to prompt the user if appropriate. If
-   * you are looking to provide an {@link InstallationMethod}, please use {@link
-   * WebExtensionController#install(String, String)}
+   * PromptDelegate#installPromptRequest(GeckoBundle, EventCallback)}, you can use this method to
+   * prompt the user if appropriate. If you are looking to provide an {@link InstallationMethod},
+   * please use {@link WebExtensionController#install(String, String)}
    *
    * @param uri URI to the extension's <code>.xpi</code> package. This can be a remote <code>https:
    *     </code> URI or a local <code>file:</code> or <code>resource:</code> URI. Note: the app
@@ -675,7 +735,7 @@ public class WebExtensionController {
    *     exceptionally with a {@link WebExtension.InstallException InstallException} that will
    *     contain the relevant error code in {@link WebExtension.InstallException#code
    *     InstallException#code}.
-   * @see PromptDelegate#installPrompt
+   * @see PromptDelegate#installPromptRequest
    * @see WebExtension.InstallException.ErrorCodes
    * @see WebExtension#metaData
    */
@@ -687,7 +747,11 @@ public class WebExtensionController {
 
   /** The method used by the embedder to install the {@link WebExtension}. */
   @Retention(RetentionPolicy.SOURCE)
-  @StringDef({INSTALLATION_METHOD_MANAGER, INSTALLATION_METHOD_FROM_FILE})
+  @StringDef({
+    INSTALLATION_METHOD_MANAGER,
+    INSTALLATION_METHOD_FROM_FILE,
+    INSTALLATION_METHOD_ONBOARDING
+  })
   public @interface InstallationMethod {};
 
   /** Indicates the {@link WebExtension} was installed using from the embedder's add-ons manager. */
@@ -696,6 +760,9 @@ public class WebExtensionController {
   /** Indicates the {@link WebExtension} was installed from a file. */
   public static final String INSTALLATION_METHOD_FROM_FILE = "install-from-file";
 
+  /** Indicates the {@link WebExtension} was installed from the embedder's onboarding feature. */
+  public static final String INSTALLATION_METHOD_ONBOARDING = "onboarding";
+
   /**
    * Set whether an extension should be allowed to run in private browsing or not.
    *
@@ -724,6 +791,8 @@ public class WebExtensionController {
    * @param extensionId the id of {@link WebExtension} instance to modify.
    * @param permissions the permissions to add, pass an empty array to not update.
    * @param origins the origins to add, pass an empty array to not update.
+   * @param dataCollectionPermissions the data collection permissions to add, pass an empty array to
+   *     not update.
    * @return the updated {@link WebExtension} instance.
    */
   @NonNull
@@ -731,11 +800,13 @@ public class WebExtensionController {
   public GeckoResult<WebExtension> addOptionalPermissions(
       final @NonNull String extensionId,
       @NonNull final String[] permissions,
-      @NonNull final String[] origins) {
-    final GeckoBundle bundle = new GeckoBundle(3);
+      @NonNull final String[] origins,
+      @NonNull final String[] dataCollectionPermissions) {
+    final GeckoBundle bundle = new GeckoBundle(4);
     bundle.putString("extensionId", extensionId);
     bundle.putStringArray("permissions", permissions);
     bundle.putStringArray("origins", origins);
+    bundle.putStringArray("dataCollectionPermissions", dataCollectionPermissions);
 
     return EventDispatcher.getInstance()
         .queryBundle("GeckoView:WebExtension:AddOptionalPermissions", bundle)
@@ -749,6 +820,8 @@ public class WebExtensionController {
    * @param extensionId the id of {@link WebExtension} instance to modify.
    * @param permissions the permissions to remove, pass an empty array to not update.
    * @param origins the origins to remove, pass an empty array to not update.
+   * @param dataCollectionPermissions the data collection permissions to remove, pass an array to
+   *     not update.
    * @return the updated {@link WebExtension} instance.
    */
   @NonNull
@@ -756,11 +829,13 @@ public class WebExtensionController {
   public GeckoResult<WebExtension> removeOptionalPermissions(
       final @NonNull String extensionId,
       @NonNull final String[] permissions,
-      @NonNull final String[] origins) {
-    final GeckoBundle bundle = new GeckoBundle(3);
+      @NonNull final String[] origins,
+      @NonNull final String[] dataCollectionPermissions) {
+    final GeckoBundle bundle = new GeckoBundle(4);
     bundle.putString("extensionId", extensionId);
     bundle.putStringArray("permissions", permissions);
     bundle.putStringArray("origins", origins);
+    bundle.putStringArray("dataCollectionPermissions", dataCollectionPermissions);
 
     return EventDispatcher.getInstance()
         .queryBundle("GeckoView:WebExtension:RemoveOptionalPermissions", bundle)
@@ -974,7 +1049,7 @@ public class WebExtensionController {
    * @return A {@link GeckoResult} that will complete when the update process finishes. If an update
    *     is found and installed successfully, the GeckoResult will return the updated {@link
    *     WebExtension}. If no update is available, null will be returned. If the updated extension
-   *     requires new permissions, the {@link PromptDelegate#installPrompt} will be called.
+   *     requires new permissions, the {@link PromptDelegate#installPromptRequest} will be called.
    * @see PromptDelegate#updatePrompt
    */
   @AnyThread
@@ -1018,7 +1093,7 @@ public class WebExtensionController {
     Log.d(LOGTAG, "handleMessage " + event);
 
     if ("GeckoView:WebExtension:InstallPrompt".equals(event)) {
-      installPrompt(bundle, callback);
+      installPromptRequest(bundle, callback);
       return;
     } else if ("GeckoView:WebExtension:UpdatePrompt".equals(event)) {
       updatePrompt(bundle, callback);
@@ -1143,7 +1218,7 @@ public class WebExtensionController {
             });
   }
 
-  private void installPrompt(final GeckoBundle message, final EventCallback callback) {
+  private void installPromptRequest(final GeckoBundle message, final EventCallback callback) {
     final GeckoBundle extensionBundle = message.getBundle("extension");
     if (extensionBundle == null
         || !extensionBundle.containsKey("webExtensionId")
@@ -1164,39 +1239,37 @@ public class WebExtensionController {
       return;
     }
 
-    final GeckoResult<AllowOrDeny> promptResponse = mPromptDelegate.onInstallPrompt(extension);
+    final GeckoResult<WebExtension.PermissionPromptResponse> promptResponse =
+        mPromptDelegate.onInstallPromptRequest(
+            extension,
+            message.getStringArray("permissions"),
+            message.getStringArray("origins"),
+            message.getStringArray("dataCollectionPermissions"));
+
     if (promptResponse == null) {
       return;
     }
 
     callback.resolveTo(
         promptResponse.map(
-            allowOrDeny -> {
-              final GeckoBundle response = new GeckoBundle(1);
-              response.putBoolean("allow", AllowOrDeny.ALLOW.equals(allowOrDeny));
+            userResponse -> {
+              final GeckoBundle response = new GeckoBundle(3);
+              response.putBoolean("allow", userResponse.isPermissionsGranted);
+              response.putBoolean("privateBrowsingAllowed", userResponse.isPrivateModeGranted);
+              response.putBoolean(
+                  "isTechnicalAndInteractionDataGranted",
+                  userResponse.isTechnicalAndInteractionDataGranted);
               return response;
             }));
   }
 
   private void updatePrompt(final GeckoBundle message, final EventCallback callback) {
-    final GeckoBundle currentBundle = message.getBundle("currentlyInstalled");
-    final GeckoBundle updatedBundle = message.getBundle("updatedExtension");
+    final WebExtension currentExtension =
+        new WebExtension(mDelegateControllerProvider, message.getBundle("currentlyInstalled"));
+    final WebExtension updatedExtension =
+        new WebExtension(mDelegateControllerProvider, message.getBundle("updatedExtension"));
     final String[] newPermissions = message.getStringArray("newPermissions");
     final String[] newOrigins = message.getStringArray("newOrigins");
-    if (currentBundle == null || updatedBundle == null) {
-      if (BuildConfig.DEBUG_BUILD) {
-        throw new RuntimeException("Missing bundle");
-      }
-
-      Log.e(LOGTAG, "Missing bundle");
-      return;
-    }
-
-    final WebExtension currentExtension =
-        new WebExtension(mDelegateControllerProvider, currentBundle);
-
-    final WebExtension updatedExtension =
-        new WebExtension(mDelegateControllerProvider, updatedBundle);
 
     if (mPromptDelegate == null) {
       Log.e(
@@ -1205,9 +1278,21 @@ public class WebExtensionController {
       return;
     }
 
-    final GeckoResult<AllowOrDeny> promptResponse =
+    // TODO - Bug 1974744: remove the first call when we remove the deprecated
+    // `onUpdatePrompt` method since this has been done to preserve backward
+    // compatibility.
+    GeckoResult<AllowOrDeny> promptResponse =
         mPromptDelegate.onUpdatePrompt(
             currentExtension, updatedExtension, newPermissions, newOrigins);
+
+    if (promptResponse == null) {
+      final String[] newDataCollectionPermissions =
+          message.getStringArray("newDataCollectionPermissions");
+      promptResponse =
+          mPromptDelegate.onUpdatePrompt(
+              updatedExtension, newPermissions, newOrigins, newDataCollectionPermissions);
+    }
+
     if (promptResponse == null) {
       return;
     }
@@ -1234,8 +1319,22 @@ public class WebExtensionController {
     final String[] permissions =
         message.bundle.getBundle("permissions").getStringArray("permissions");
     final String[] origins = message.bundle.getBundle("permissions").getStringArray("origins");
-    final GeckoResult<AllowOrDeny> promptResponse =
+
+    // TODO - Bug 1972510: remove the first call when we remove the deprecated
+    // `onOptionalPrompt` method since this has been done to preserve backward
+    // compatibility.
+    GeckoResult<AllowOrDeny> promptResponse =
         mPromptDelegate.onOptionalPrompt(extension, permissions, origins);
+
+    if (promptResponse == null) {
+      final String[] dataCollectionPermissions =
+          message.bundle.getBundle("permissions").getStringArray("data_collection");
+
+      promptResponse =
+          mPromptDelegate.onOptionalPrompt(
+              extension, permissions, origins, dataCollectionPermissions);
+    }
+
     if (promptResponse == null) {
       return;
     }
@@ -1258,13 +1357,15 @@ public class WebExtensionController {
     final int errorCode = bundle.getInt("error");
     final GeckoBundle extensionBundle = bundle.getBundle("extension");
     WebExtension extension = null;
+    final String extensionId = bundle.getString("addonId");
     final String extensionName = bundle.getString("addonName");
+    final String extensionVersion = bundle.getString("addonVersion");
 
     if (extensionBundle != null) {
       extension = new WebExtension(mDelegateControllerProvider, extensionBundle);
     }
     mAddonManagerDelegate.onInstallationFailed(
-        extension, new InstallException(errorCode, extensionName));
+        extension, new InstallException(errorCode, extensionId, extensionName, extensionVersion));
   }
 
   private void onDisabling(final GeckoBundle bundle) {
