@@ -11,8 +11,8 @@
 
 // Globals
 
-const { TelemetryTestUtils } = ChromeUtils.import(
-  "resource://testing-common/TelemetryTestUtils.jsm"
+const { TelemetryTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TelemetryTestUtils.sys.mjs"
 );
 
 Services.prefs.setBoolPref(
@@ -319,10 +319,10 @@ add_task(async function test_history_expiration() {
   let downloadOne = await promiseNewDownload();
   let downloadTwo = await promiseNewDownload(httpUrl("interruptible.txt"));
 
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   let removeNotifications = 0;
   let downloadView = {
-    onDownloadRemoved(aDownload) {
+    onDownloadRemoved() {
       if (++removeNotifications == 2) {
         deferred.resolve();
       }
@@ -366,10 +366,10 @@ add_task(async function test_history_clear() {
   await list.add(downloadOne);
   await list.add(downloadTwo);
 
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   let removeNotifications = 0;
   let downloadView = {
-    onDownloadRemoved(aDownload) {
+    onDownloadRemoved() {
       if (++removeNotifications == 2) {
         deferred.resolve();
       }
@@ -401,7 +401,7 @@ add_task(async function test_removeFinished() {
   await list.add(downloadThree);
   await list.add(downloadFour);
 
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   let removeNotifications = 0;
   let downloadView = {
     onDownloadRemoved(aDownload) {
@@ -410,7 +410,7 @@ add_task(async function test_removeFinished() {
           aDownload == downloadTwo ||
           aDownload == downloadThree
       );
-      Assert.ok(removeNotifications < 3);
+      Assert.less(removeNotifications, 3);
       if (++removeNotifications == 3) {
         deferred.resolve();
       }
@@ -454,7 +454,7 @@ add_task(async function test_removeFinished_keepsDownloadingFile() {
   await list.add(oneDownload);
   await list.add(otherDownload);
 
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   let downloadView = {
     async onDownloadRemoved(aDownload) {
       Assert.equal(aDownload, oneDownload);
@@ -481,7 +481,7 @@ add_task(async function test_removeFinished_keepsDownloadingFile() {
   );
 
   Assert.ok(
-    await OS.File.exists(otherDownload.target.path),
+    await IOUtils.exists(otherDownload.target.path),
     "The file should not have been deleted."
   );
 });
@@ -652,7 +652,6 @@ add_task(async function test_DownloadSummary_notifications() {
  */
 add_task(async function test_downloadAddedTelemetry() {
   Services.telemetry.clearEvents();
-  Services.telemetry.setEventRecordingEnabled("downloads", true);
 
   let targetFile = getTempFile(TEST_TARGET_FILE_NAME);
 
