@@ -2,17 +2,17 @@
 /* vim: set sts=2 sw=2 et tw=80: */
 "use strict";
 
-const { AddonManager } = ChromeUtils.import(
-  "resource://gre/modules/AddonManager.jsm"
+const { AddonManager } = ChromeUtils.importESModule(
+  "resource://gre/modules/AddonManager.sys.mjs"
 );
-const { MockRegistrar } = ChromeUtils.import(
-  "resource://testing-common/MockRegistrar.jsm"
+const { MockRegistrar } = ChromeUtils.importESModule(
+  "resource://testing-common/MockRegistrar.sys.mjs"
 );
 
 const id = "uninstall_self_test@tests.mozilla.com";
 
 const manifest = {
-  applications: {
+  browser_specific_settings: {
     gecko: {
       id,
     },
@@ -38,7 +38,7 @@ const waitForUninstalled = () =>
 let promptService = {
   _response: null,
   QueryInterface: ChromeUtils.generateQI(["nsIPromptService"]),
-  confirmEx: function(...args) {
+  confirmEx: function (...args) {
     this._confirmExArgs = args;
     return this._response;
   },
@@ -48,7 +48,7 @@ AddonTestUtils.init(this);
 
 add_task(async function setup() {
   let fakePromptService = MockRegistrar.register(
-    "@mozilla.org/embedcomp/prompt-service;1",
+    "@mozilla.org/prompter;1",
     promptService
   );
   registerCleanupFunction(() => {
@@ -59,7 +59,7 @@ add_task(async function setup() {
 
 add_task(async function test_management_uninstall_no_prompt() {
   function background() {
-    browser.test.onMessage.addListener(msg => {
+    browser.test.onMessage.addListener(() => {
       browser.management.uninstallSelf();
     });
   }
@@ -82,7 +82,7 @@ add_task(async function test_management_uninstall_prompt_uninstall() {
   promptService._response = 0;
 
   function background() {
-    browser.test.onMessage.addListener(msg => {
+    browser.test.onMessage.addListener(() => {
       browser.management.uninstallSelf({ showConfirmDialog: true });
     });
   }
@@ -114,7 +114,7 @@ add_task(async function test_management_uninstall_prompt_keep() {
   promptService._response = 1;
 
   function background() {
-    browser.test.onMessage.addListener(async msg => {
+    browser.test.onMessage.addListener(async () => {
       await browser.test.assertRejects(
         browser.management.uninstallSelf({ showConfirmDialog: true }),
         "User cancelled uninstall of extension",
