@@ -3,17 +3,17 @@ http://creativecommons.org/publicdomain/zero/1.0/ */
 "use strict";
 
 const {
-  getFileHandleFromOPFS,
   MultiProgressAggregator,
   ProgressAndStatusCallbackParams,
   ProgressStatusText,
   readResponse,
-  modelToResponse,
   URLChecker,
-  removeFromOPFS,
   RejectionType,
   BlockListManager,
   RemoteSettingsManager,
+  isAddonEngineId,
+  addonIdToEngineId,
+  engineIdToAddonId,
 } = ChromeUtils.importESModule("chrome://global/content/ml/Utils.sys.mjs");
 
 /**
@@ -450,53 +450,6 @@ add_task(async function test_multi_aggregator() {
   Assert.equal(numDone, 1, "Done status should be received");
 });
 
-/**
- * Test modelToResponse function.
- */
-add_task(async function test_ml_utils_model_to_response() {
-  const modelPath = "test.txt";
-  await getFileHandleFromOPFS(modelPath, { create: true });
-
-  registerCleanupFunction(async () => {
-    await removeFromOPFS(modelPath);
-  });
-
-  const cases = [
-    {
-      model: modelPath,
-      headers: null,
-      expected: {},
-      msg: "valid response with no headers",
-    },
-    {
-      model: modelPath,
-      headers: { some: "header" },
-      expected: { some: "header" },
-      msg: "valid response",
-    },
-
-    {
-      model: modelPath,
-      headers: { some: "header", int: 1234 },
-      expected: { some: "header", int: "1234" },
-      msg: "valid response with ints conversion",
-    },
-    {
-      model: modelPath,
-      headers: { some: null, int: 1234 },
-      expected: { int: "1234" },
-      msg: "valid response with null keys ignored",
-    },
-  ];
-
-  for (const case_ of cases) {
-    const response = await modelToResponse(case_.model, case_.headers);
-    for (const [key, value] of Object.entries(case_.expected)) {
-      Assert.deepEqual(response.headers.get(key), value, case_.message);
-    }
-  }
-});
-
 /**
  * Tests the URLChecker class
  *
@@ -1105,3 +1058,46 @@ add_task(async function testBlockListManagerWithRS() {
 
   RemoteSettingsManager.removeMocks();
 });
+
+add_task(function test_addon_engine_id_utilities() {
+  const prefix = "ML-ENGINE-";
+
+  // Valid addon ID
+  const addonId = "custom-addon";
+  const engineId = addonIdToEngineId(addonId);
+
+  Assert.equal(
+    engineId,
+    `${prefix}${addonId}`,
+    "addonIdToEngineId should add the correct prefix"
+  );
+  Assert.ok(
+    isAddonEngineId(engineId),
+    "isAddonEngineId should detect prefixed engine ID"
+  );
+  Assert.equal(
+    engineIdToAddonId(engineId),
+    addonId,
+    "engineIdToAddonId should return original addon ID"
+  );
+
+  // Invalid engine ID
+  const invalidEngineId = "ENGINE-custom-addon";
+  Assert.ok(
+    !isAddonEngineId(invalidEngineId),
+    "isAddonEngineId should reject non-prefixed engine ID"
+  );
+  Assert.equal(
+    engineIdToAddonId(invalidEngineId),
+    null,
+    "engineIdToAddonId should return null for invalid ID"
+  );
+
+  // Edge case: empty string
+  Assert.ok(!isAddonEngineId(""), "isAddonEngineId should reject empty string");
+  Assert.equal(
+    engineIdToAddonId(""),
+    null,
+    "engineIdToAddonId should return null for empty string"
+  );
+});
