@@ -28,8 +28,9 @@ add_task(async function testPolicyDisablesNimbus() {
     "Policy engine is active"
   );
 
-  const loader = ExperimentFakes.rsLoader();
+  const loader = NimbusTestUtils.stubs.rsLoader();
   const manager = loader.manager;
+  await manager.store.init();
   await manager.onStartup();
 
   Assert.ok(!manager.studiesEnabled, "ExperimentManager is disabled");
