@@ -687,20 +687,23 @@ NavHistoryResultObserver.prototype = {
 function checkBookmarkObject(info) {
   do_check_valid_places_guid(info.guid);
   do_check_valid_places_guid(info.parentGuid);
-  Assert.ok(typeof info.index == "number", "index should be a number");
-  Assert.ok(
-    info.dateAdded.constructor.name == "Date",
+  Assert.equal(typeof info.index, "number", "index should be a number");
+  Assert.equal(
+    info.dateAdded.constructor.name,
+    "Date",
     "dateAdded should be a Date"
   );
-  Assert.ok(
-    info.lastModified.constructor.name == "Date",
+  Assert.equal(
+    info.lastModified.constructor.name,
+    "Date",
     "lastModified should be a Date"
   );
-  Assert.ok(
-    info.lastModified >= info.dateAdded,
+  Assert.greaterOrEqual(
+    info.lastModified,
+    info.dateAdded,
     "lastModified should never be smaller than dateAdded"
   );
-  Assert.ok(typeof info.type == "number", "type should be a number");
+  Assert.equal(typeof info.type, "number", "type should be a number");
 }
 
 /**
@@ -734,38 +737,6 @@ function sortBy(array, prop) {
   return array.sort((a, b) => compareAscending(a[prop], b[prop]));
 }
 
-function getFaviconUrlForPage(page, width = 0) {
-  let pageURI =
-    page instanceof Ci.nsIURI ? page : NetUtil.newURI(new URL(page).href);
-  return new Promise((resolve, reject) => {
-    PlacesUtils.favicons.getFaviconURLForPage(
-      pageURI,
-      iconURI => {
-        if (iconURI) {
-          resolve(iconURI.spec);
-        } else {
-          reject("Unable to find an icon for " + pageURI.spec);
-        }
-      },
-      width
-    );
-  });
-}
-
-function getFaviconDataForPage(page, width = 0) {
-  let pageURI =
-    page instanceof Ci.nsIURI ? page : NetUtil.newURI(new URL(page).href);
-  return new Promise(resolve => {
-    PlacesUtils.favicons.getFaviconDataForPage(
-      pageURI,
-      (iconUri, len, data, mimeType) => {
-        resolve({ data, mimeType });
-      },
-      width
-    );
-  });
-}
-
 /**
  * Asynchronously compares contents from 2 favicon urls.
  */
