@@ -6,292 +6,257 @@
 // that aren't initialized outside of a XUL app environment like AddonManager
 // and the "@mozilla.org/xre/app-info;1" component.
 
-const { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+const { Troubleshoot } = ChromeUtils.importESModule(
+  "resource://gre/modules/Troubleshoot.sys.mjs"
 );
-const { Troubleshoot } = ChromeUtils.import(
-  "resource://gre/modules/Troubleshoot.jsm"
+const { sinon } = ChromeUtils.importESModule(
+  "resource://testing-common/Sinon.sys.mjs"
 );
-const { sinon } = ChromeUtils.import("resource://testing-common/Sinon.jsm");
 
-const { FeatureGate } = ChromeUtils.import(
-  "resource://featuregates/FeatureGate.jsm"
+const { PreferenceExperiments } = ChromeUtils.importESModule(
+  "resource://normandy/lib/PreferenceExperiments.sys.mjs"
 );
-const { PreferenceExperiments } = ChromeUtils.import(
-  "resource://normandy/lib/PreferenceExperiments.jsm"
+const { PreferenceRollouts } = ChromeUtils.importESModule(
+  "resource://normandy/lib/PreferenceRollouts.sys.mjs"
 );
-const { PreferenceRollouts } = ChromeUtils.import(
-  "resource://normandy/lib/PreferenceRollouts.jsm"
+const { AddonStudies } = ChromeUtils.importESModule(
+  "resource://normandy/lib/AddonStudies.sys.mjs"
 );
-const { AddonStudies } = ChromeUtils.import(
-  "resource://normandy/lib/AddonStudies.jsm"
-);
-const { NormandyTestUtils } = ChromeUtils.import(
-  "resource://testing-common/NormandyTestUtils.jsm"
+const { NormandyTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/NormandyTestUtils.sys.mjs"
 );
 
 NormandyTestUtils.init({ Assert });
 
-function test() {
-  waitForExplicitFinish();
-  function doNextTest() {
-    if (!tests.length) {
-      finish();
-      return;
-    }
-    tests.shift()(doNextTest);
+add_task(async function snapshotSchema() {
+  let snapshot = await Troubleshoot.snapshot();
+  try {
+    validateObject(snapshot, SNAPSHOT_SCHEMA);
+    ok(true, "The snapshot should conform to the schema.");
+  } catch (err) {
+    ok(false, "Schema mismatch, " + err);
   }
-  doNextTest();
-}
-
-registerCleanupFunction(function() {
-  // Troubleshoot.jsm is imported into the global scope -- the window -- above.
-  // If it's not deleted, it outlives the test and is reported as a leak.
-  delete window.Troubleshoot;
 });
 
-var tests = [
-  function snapshotSchema(done) {
-    Troubleshoot.snapshot(function(snapshot) {
-      try {
-        validateObject(snapshot, SNAPSHOT_SCHEMA);
-        ok(true, "The snapshot should conform to the schema.");
-      } catch (err) {
-        ok(false, "Schema mismatch, " + err);
-      }
-      done();
-    });
-  },
+add_task(async function modifiedPreferences() {
+  let prefs = [
+    "javascript.troubleshoot",
+    "troubleshoot.foo",
+    "network.proxy.troubleshoot",
+    "print.print_to_filename",
+  ];
+  prefs.forEach(function (p) {
+    Services.prefs.setBoolPref(p, true);
+    is(Services.prefs.getBoolPref(p), true, "The pref should be set: " + p);
+  });
+  Services.prefs.setCharPref("dom.push.userAgentID", "testvalue");
+  let snapshot = await Troubleshoot.snapshot();
+  let p = snapshot.modifiedPreferences;
+  is(
+    p["javascript.troubleshoot"],
+    true,
+    "The pref should be present because it's in the allowed prefs " +
+      "and not in the pref regexes that are disallowed."
+  );
+  ok(
+    !("troubleshoot.foo" in p),
+    "The pref should be absent because it's not in the allowed prefs."
+  );
+  ok(
+    !("network.proxy.troubleshoot" in p),
+    "The pref should be absent because it's in the pref regexes " +
+      "that are disallowed."
+  );
+  ok(
+    !("dom.push.userAgentID" in p),
+    "The pref should be absent because it's in the pref regexes " +
+      "that are disallowed."
+  );
+  ok(
+    !("print.print_to_filename" in p),
+    "The pref should be absent because it's not in the allowed prefs."
+  );
+  prefs.forEach(p => Services.prefs.deleteBranch(p));
+  Services.prefs.clearUserPref("dom.push.userAgentID");
+});
 
-  async function experimentalFeatures(done) {
-    let featureGates = await FeatureGate.all();
-    ok(featureGates.length, "Should be at least one FeatureGate");
+add_task(async function unicodePreferences() {
+  let name = "font.name.sans-serif.x-western";
+  let utf8Value = "\xc4\x8capk\xc5\xafv Krasopis";
+  let unicodeValue = "\u010Capk\u016Fv Krasopis";
 
-    Troubleshoot.snapshot(snapshot => {
-      for (let i = 0; i < snapshot.experimentalFeatures.length; i++) {
-        let experimentalFeature = snapshot.experimentalFeatures[i];
-        is(
-          experimentalFeature[0],
-          featureGates[i].title,
-          "The first item in the array should be the title's l10n-id of the FeatureGate"
-        );
-        is(
-          experimentalFeature[1],
-          featureGates[i].preference,
-          "The second item in the array should be the preference name for the FeatureGate"
-        );
-        is(
-          experimentalFeature[2],
-          Services.prefs.getBoolPref(featureGates[i].preference),
-          "The third item in the array should be the preference value of the FeatureGate"
-        );
-      }
-      done();
-    });
-  },
+  // set/getCharPref work with 8bit strings (utf8)
+  Services.prefs.setCharPref(name, utf8Value);
 
-  function modifiedPreferences(done) {
-    let prefs = [
-      "javascript.troubleshoot",
-      "troubleshoot.foo",
-      "network.proxy.troubleshoot",
-      "print.print_to_filename",
-    ];
-    prefs.forEach(function(p) {
-      Services.prefs.setBoolPref(p, true);
-      is(Services.prefs.getBoolPref(p), true, "The pref should be set: " + p);
-    });
-    Services.prefs.setCharPref("dom.push.userAgentID", "testvalue");
-    Troubleshoot.snapshot(function(snapshot) {
-      let p = snapshot.modifiedPreferences;
-      is(
-        p["javascript.troubleshoot"],
-        true,
-        "The pref should be present because it's in the allowed prefs " +
-          "and not in the pref regexes that are disallowed."
-      );
-      ok(
-        !("troubleshoot.foo" in p),
-        "The pref should be absent because it's not in the allowed prefs."
-      );
-      ok(
-        !("network.proxy.troubleshoot" in p),
-        "The pref should be absent because it's in the pref regexes " +
-          "that are disallowed."
-      );
-      ok(
-        !("dom.push.userAgentID" in p),
-        "The pref should be absent because it's in the pref regexes " +
-          "that are disallowed."
-      );
-      ok(
-        !("print.print_to_filename" in p),
-        "The pref should be absent because it's not in the allowed prefs."
-      );
-      prefs.forEach(p => Services.prefs.deleteBranch(p));
-      Services.prefs.clearUserPref("dom.push.userAgentID");
-      done();
-    });
-  },
+  let snapshot = await Troubleshoot.snapshot();
+  let p = snapshot.modifiedPreferences;
+  is(p[name], unicodeValue, "The pref should have correct Unicode value.");
+  Services.prefs.deleteBranch(name);
+});
 
-  function unicodePreferences(done) {
-    let name = "font.name.sans-serif.x-western";
-    let utf8Value = "\xc4\x8capk\xc5\xafv Krasopis";
-    let unicodeValue = "\u010Capk\u016Fv Krasopis";
+add_task(async function printingPreferences() {
+  let prefs = [
+    "javascript.print_to_filename",
+    "print.print_bgimages",
+    "print.print_to_filename",
+  ];
+  prefs.forEach(function (p) {
+    Services.prefs.setBoolPref(p, true);
+    is(Services.prefs.getBoolPref(p), true, "The pref should be set: " + p);
+  });
+  let snapshot = await Troubleshoot.snapshot();
+  let p = snapshot.printingPreferences;
+  is(p["print.print_bgimages"], true, "The pref should be present");
+  ok(
+    !("print.print_to_filename" in p),
+    "The pref should not be present (sensitive)"
+  );
+  ok(
+    !("javascript.print_to_filename" in p),
+    "The pref should be absent because it's not a print pref."
+  );
+  prefs.forEach(p => Services.prefs.deleteBranch(p));
+});
 
-    // set/getCharPref work with 8bit strings (utf8)
-    Services.prefs.setCharPref(name, utf8Value);
+add_task(function normandy() {
+  const {
+    preferenceStudyFactory,
+    branchedAddonStudyFactory,
+    preferenceRolloutFactory,
+  } = NormandyTestUtils.factories;
 
-    Troubleshoot.snapshot(function(snapshot) {
-      let p = snapshot.modifiedPreferences;
-      is(p[name], unicodeValue, "The pref should have correct Unicode value.");
-      Services.prefs.deleteBranch(name);
-      done();
-    });
-  },
-
-  function printingPreferences(done) {
-    let prefs = [
-      "javascript.print_to_filename",
-      "print.print_bgimages",
-      "print.print_to_filename",
-    ];
-    prefs.forEach(function(p) {
-      Services.prefs.setBoolPref(p, true);
-      is(Services.prefs.getBoolPref(p), true, "The pref should be set: " + p);
-    });
-    Troubleshoot.snapshot(function(snapshot) {
-      let p = snapshot.printingPreferences;
-      is(p["print.print_bgimages"], true, "The pref should be present");
-      ok(
-        !("print.print_to_filename" in p),
-        "The pref should not be present (sensitive)"
-      );
-      ok(
-        !("javascript.print_to_filename" in p),
-        "The pref should be absent because it's not a print pref."
-      );
-      prefs.forEach(p => Services.prefs.deleteBranch(p));
-      done();
-    });
-  },
-
-  function normandy(done) {
-    const {
-      preferenceStudyFactory,
-      branchedAddonStudyFactory,
-      preferenceRolloutFactory,
-    } = NormandyTestUtils.factories;
-
-    NormandyTestUtils.decorate(
-      PreferenceExperiments.withMockExperiments([
-        preferenceStudyFactory({
-          userFacingName: "Test Pref Study B",
-          branch: "test-branch-pref",
-        }),
-        preferenceStudyFactory({
-          userFacingName: "Test Pref Study A",
-          branch: "test-branch-pref",
-        }),
-      ]),
-      AddonStudies.withStudies([
-        branchedAddonStudyFactory({
-          userFacingName: "Test Addon Study B",
-          branch: "test-branch-addon",
-        }),
-        branchedAddonStudyFactory({
-          userFacingName: "Test Addon Study A",
-          branch: "test-branch-addon",
-        }),
-      ]),
-      PreferenceRollouts.withTestMock({
-        rollouts: [
-          preferenceRolloutFactory({
-            statue: "ACTIVE",
-            slug: "test-pref-rollout-b",
-          }),
-          preferenceRolloutFactory({
-            statue: "ACTIVE",
-            slug: "test-pref-rollout-a",
-          }),
-        ],
+  return NormandyTestUtils.decorate(
+    PreferenceExperiments.withMockExperiments([
+      preferenceStudyFactory({
+        userFacingName: "Test Pref Study B",
+        branch: "test-branch-pref",
       }),
-      async function testNormandyInfoInTroubleshooting({
-        prefExperiments,
-        addonStudies,
-        prefRollouts,
-      }) {
-        await new Promise(resolve => {
-          Troubleshoot.snapshot(function(snapshot) {
-            let info = snapshot.normandy;
-            // The order should be flipped, since each category is sorted by slug.
-            Assert.deepEqual(
-              info.prefStudies,
-              [prefExperiments[1], prefExperiments[0]],
-              "prefs studies should exist in the right order"
-            );
-            Assert.deepEqual(
-              info.addonStudies,
-              [addonStudies[1], addonStudies[0]],
-              "addon studies should exist in the right order"
-            );
-            Assert.deepEqual(
-              info.prefRollouts,
-              [prefRollouts[1], prefRollouts[0]],
-              "pref rollouts should exist in the right order"
-            );
-            resolve();
-          });
-        });
+      preferenceStudyFactory({
+        userFacingName: "Test Pref Study A",
+        branch: "test-branch-pref",
+      }),
+    ]),
+    AddonStudies.withStudies([
+      branchedAddonStudyFactory({
+        userFacingName: "Test Addon Study B",
+        branch: "test-branch-addon",
+      }),
+      branchedAddonStudyFactory({
+        userFacingName: "Test Addon Study A",
+        branch: "test-branch-addon",
+      }),
+    ]),
+    PreferenceRollouts.withTestMock({
+      rollouts: [
+        preferenceRolloutFactory({
+          statue: "ACTIVE",
+          slug: "test-pref-rollout-b",
+        }),
+        preferenceRolloutFactory({
+          statue: "ACTIVE",
+          slug: "test-pref-rollout-a",
+        }),
+      ],
+    }),
+    async function testNormandyInfoInTroubleshooting({
+      prefExperiments,
+      addonStudies,
+      prefRollouts,
+    }) {
+      let snapshot = await Troubleshoot.snapshot();
+      let info = snapshot.normandy;
+      // The order should be flipped, since each category is sorted by slug.
+      Assert.deepEqual(
+        info.prefStudies,
+        [prefExperiments[1], prefExperiments[0]],
+        "prefs studies should exist in the right order"
+      );
+      Assert.deepEqual(
+        info.addonStudies,
+        [addonStudies[1], addonStudies[0]],
+        "addon studies should exist in the right order"
+      );
+      Assert.deepEqual(
+        info.prefRollouts,
+        [prefRollouts[1], prefRollouts[0]],
+        "pref rollouts should exist in the right order"
+      );
+    }
+  )();
+});
+
+add_task(function normandyErrorHandling() {
+  return NormandyTestUtils.decorate(
+    NormandyTestUtils.withStub(PreferenceExperiments, "getAllActive", {
+      returnValue: Promise.reject("Expected error - PreferenceExperiments"),
+    }),
+    NormandyTestUtils.withStub(AddonStudies, "getAllActive", {
+      returnValue: Promise.reject("Expected error - AddonStudies"),
+    }),
+    NormandyTestUtils.withStub(PreferenceRollouts, "getAllActive", {
+      returnValue: Promise.reject("Expected error - PreferenceRollouts"),
+    }),
+    async function testNormandyErrorHandling() {
+      let consoleEndFn = TestUtils.listenForConsoleMessages();
+      let snapshot = await Troubleshoot.snapshot();
+      let info = snapshot.normandy;
+      Assert.deepEqual(
+        info.prefStudies,
+        [],
+        "prefs studies should be an empty list if there is an error"
+      );
+      Assert.deepEqual(
+        info.addonStudies,
+        [],
+        "addon studies should be an empty list if there is an error"
+      );
+      Assert.deepEqual(
+        info.prefRollouts,
+        [],
+        "pref rollouts should be an empty list if there is an error"
+      );
+      let msgs = await consoleEndFn();
+      let expectedSet = new Set([
+        /Expected error - PreferenceExperiments/,
+        /Expected error - AddonStudies/,
+        /Expected error - PreferenceRollouts/,
+      ]);
+
+      for (let msg of msgs) {
+        msg = msg.wrappedJSObject;
+        if (msg.level != "error") {
+          continue;
+        }
+
+        let msgContents = msg.arguments[0];
+        for (let expected of expectedSet) {
+          if (expected.test(msgContents)) {
+            expectedSet.delete(expected);
+            break;
+          }
+        }
       }
-    )().then(done);
-  },
 
-  function normandyErrorHandling(done) {
-    NormandyTestUtils.decorate(
-      NormandyTestUtils.withStub(PreferenceExperiments, "getAllActive", {
-        returnValue: Promise.reject("Expected error - PreferenceExperiments"),
-      }),
-      NormandyTestUtils.withStub(AddonStudies, "getAllActive", {
-        returnValue: Promise.reject("Expected error - AddonStudies"),
-      }),
-      NormandyTestUtils.withStub(PreferenceRollouts, "getAllActive", {
-        returnValue: Promise.reject("Expected error - PreferenceRollouts"),
-      }),
-      NormandyTestUtils.withConsoleSpy(),
-      async function testNormandyErrorHandling({ consoleSpy }) {
-        await new Promise(resolve => {
-          Troubleshoot.snapshot(snapshot => {
-            let info = snapshot.normandy;
-            Assert.deepEqual(
-              info.prefStudies,
-              [],
-              "prefs studies should be an empty list if there is an error"
-            );
-            Assert.deepEqual(
-              info.addonStudies,
-              [],
-              "addon studies should be an empty list if there is an error"
-            );
-            Assert.deepEqual(
-              info.prefRollouts,
-              [],
-              "pref rollouts should be an empty list if there is an error"
-            );
+      Assert.equal(
+        expectedSet.size,
+        0,
+        "Should have no messages left in the expected set"
+      );
+    }
+  )();
+});
 
-            consoleSpy.assertAtLeast([
-              /Expected error - PreferenceExperiments/,
-              /Expected error - AddonStudies/,
-              /Expected error - PreferenceRollouts/,
-            ]);
-
-            resolve();
-          });
-        });
-      }
-    )().then(done);
-  },
-];
+add_task(async function themes() {
+  let snapshot = await Troubleshoot.snapshot();
+  let foundTheme = false;
+  for (let addon of snapshot.addons) {
+    if (addon.type == "theme") {
+      foundTheme = true;
+      break;
+    }
+  }
+  ok(foundTheme, "found a theme in the addons list");
+});
 
 // This is inspired by JSON Schema, or by the example on its Wikipedia page
 // anyway.
@@ -327,6 +292,9 @@ const SNAPSHOT_SCHEMA = {
           required: true,
           type: "string",
         },
+        osTheme: {
+          type: "string",
+        },
         rosetta: {
           required: false,
           type: "boolean",
@@ -377,6 +345,16 @@ const SNAPSHOT_SCHEMA = {
         safeMode: {
           type: "boolean",
         },
+        memorySizeBytes: {
+          type: "number",
+        },
+        diskAvailableBytes: {
+          type: "number",
+        },
+        pointingDevices: {
+          required: false,
+          type: "array",
+        },
       },
     },
     crashes: {
@@ -436,6 +414,10 @@ const SNAPSHOT_SCHEMA = {
             required: true,
             type: "boolean",
           },
+          locationName: {
+            required: true,
+            type: "string",
+          },
         },
       },
     },
@@ -457,27 +439,6 @@ const SNAPSHOT_SCHEMA = {
         },
       },
     },
-    features: {
-      required: true,
-      type: "array",
-      items: {
-        type: "object",
-        properties: {
-          name: {
-            required: true,
-            type: "string",
-          },
-          version: {
-            required: true,
-            type: "string",
-          },
-          id: {
-            required: true,
-            type: "string",
-          },
-        },
-      },
-    },
     processes: {
       required: true,
       type: "object",
@@ -492,10 +453,6 @@ const SNAPSHOT_SCHEMA = {
         },
       },
     },
-    experimentalFeatures: {
-      required: true,
-      type: "array",
-    },
     environmentVariables: {
       required: true,
       type: "object",
@@ -520,6 +477,51 @@ const SNAPSHOT_SCHEMA = {
           required: false,
           type: "boolean",
         },
+        "media.utility-process.enabled": {
+          required: true,
+          type: "boolean",
+        },
+        "extensions.formautofill.creditCards.os-auth.locked.enabled": {
+          required: false,
+          type: "boolean",
+        },
+        "signon.management.page.os-auth.locked.enabled": {
+          required: false,
+          type: "boolean",
+        },
+      },
+    },
+    places: {
+      required: true,
+      type: "array",
+      items: {
+        type: "object",
+        items: {
+          entity: {
+            required: true,
+            type: "string",
+          },
+          count: {
+            required: true,
+            type: "number",
+          },
+          sizeBytes: {
+            required: true,
+            type: "number",
+          },
+          sizePerc: {
+            required: true,
+            type: "number",
+          },
+          efficiencyPerc: {
+            required: true,
+            type: "number",
+          },
+          sequentialityPerc: {
+            required: true,
+            type: "number",
+          },
+        },
       },
     },
     graphics: {
@@ -534,15 +536,18 @@ const SNAPSHOT_SCHEMA = {
           required: true,
           type: "number",
         },
+        graphicsDevicePixelRatios: {
+          type: "array",
+          items: {
+            type: "number",
+          },
+        },
         windowLayerManagerType: {
           type: "string",
         },
         windowLayerManagerRemote: {
           type: "boolean",
         },
-        windowUsingAdvancedLayers: {
-          type: "boolean",
-        },
         numAcceleratedWindowsMessage: {
           type: "object",
           properties: {
@@ -622,18 +627,6 @@ const SNAPSHOT_SCHEMA = {
         directWriteVersion: {
           type: "string",
         },
-        usesTiling: {
-          type: "boolean",
-        },
-        contentUsesTiling: {
-          type: "boolean",
-        },
-        offMainThreadPaintEnabled: {
-          type: "boolean",
-        },
-        offMainThreadPaintWorkerCount: {
-          type: "number",
-        },
         clearTypeParameters: {
           type: "string",
         },
@@ -667,20 +660,19 @@ const SNAPSHOT_SCHEMA = {
         webgl2WSIInfo: {
           type: "string",
         },
+        webgpuDefaultAdapter: {
+          type: "object",
+        },
+        webgpuFallbackAdapter: {
+          type: "object",
+        },
         info: {
           type: "object",
         },
         failures: {
-          type: "object",
-          properties: {
-            key: {
-              required: true,
-              type: "string",
-            },
-            args: {
-              required: false,
-              type: "object",
-            },
+          type: "array",
+          items: {
+            type: "string",
           },
         },
         indices: {
@@ -717,6 +709,9 @@ const SNAPSHOT_SCHEMA = {
         desktopEnvironment: {
           type: "string",
         },
+        supportFontDetermination: {
+          type: "string",
+        },
       },
     },
     media: {
@@ -865,6 +860,10 @@ const SNAPSHOT_SCHEMA = {
             },
           },
         },
+        codecSupportInfo: {
+          required: false,
+          type: "string",
+        },
       },
     },
     accessibility: {
@@ -1009,9 +1008,13 @@ const SNAPSHOT_SCHEMA = {
           type: "number",
         },
         contentWin32kLockdownState: {
-          required: AppConstants.MOZ_SANDBOX,
+          required: AppConstants.MOZ_SANDBOX && AppConstants.platform == "win",
           type: "string",
         },
+        supportSandboxGpuLevel: {
+          required: AppConstants.MOZ_SANDBOX && AppConstants.platform == "win",
+          type: "number",
+        },
         syscallLog: {
           required: AppConstants.platform == "linux",
           type: "array",
@@ -1121,7 +1124,7 @@ const SNAPSHOT_SCHEMA = {
     remoteAgent: {
       type: "object",
       properties: {
-        listening: {
+        running: {
           required: true,
           type: "boolean",
         },
@@ -1184,7 +1187,7 @@ const SNAPSHOT_SCHEMA = {
           },
           required: true,
         },
-        remoteConfigs: {
+        nimbusRollouts: {
           type: "array",
           items: {
             type: "object",
@@ -1196,6 +1199,75 @@ const SNAPSHOT_SCHEMA = {
         },
       },
     },
+    remoteSettings: {
+      type: "object",
+      additionalProperties: true,
+      properties: {
+        isSynchronizationBroken: {
+          required: true,
+          type: "boolean",
+        },
+        lastCheck: {
+          required: true,
+          type: "string",
+        },
+        localTimestamp: {
+          required: false,
+          type: ["number", "null"],
+        },
+        history: {
+          required: true,
+          type: "object",
+          properties: {
+            "settings-sync": {
+              type: "array",
+              items: {
+                type: "object",
+                properties: {
+                  status: { type: "string", required: true },
+                  datetime: { type: "string", required: true },
+                  infos: { type: "object", required: true },
+                },
+              },
+            },
+          },
+        },
+      },
+    },
+    legacyUserStylesheets: {
+      type: "object",
+      properties: {
+        active: {
+          required: true,
+          type: "boolean",
+        },
+        types: {
+          required: true,
+          type: "array",
+        },
+      },
+    },
+    contentAnalysis: {
+      type: "object",
+      properties: {
+        active: {
+          required: true,
+          type: "boolean",
+        },
+        connected: {
+          type: "boolean",
+        },
+        agentPath: {
+          type: "string",
+        },
+        failedSignatureVerification: {
+          type: "boolean",
+        },
+        requestCount: {
+          type: "number",
+        },
+      },
+    },
   },
 };
 
@@ -1212,17 +1284,27 @@ function validateObject(obj, schema) {
   if (obj === undefined && !schema.required) {
     return;
   }
-  if (typeof schema.type != "string") {
-    throw schemaErr("'type' must be a string", schema);
+  let types = Array.isArray(schema.type) ? schema.type : [schema.type];
+  if (!types.every(elt => typeof elt == "string")) {
+    throw schemaErr("'type' must be a string or array of strings", schema);
   }
-  if (objType(obj) != schema.type) {
+  if (!types.includes(objType(obj))) {
     throw validationErr("Object is not of the expected type", obj, schema);
   }
-  let validatorFnName = "validateObject_" + schema.type;
-  if (!(validatorFnName in this)) {
-    throw schemaErr("Validator function not defined for type", schema);
+  let lastError;
+  for (let type of types) {
+    let validatorFnName = "validateObject_" + type;
+    if (!(validatorFnName in this)) {
+      throw schemaErr("Validator function not defined for type", schema);
+    }
+    try {
+      this[validatorFnName](obj, schema);
+      return;
+    } catch (e) {
+      lastError = e;
+    }
   }
-  this[validatorFnName](obj, schema);
+  throw lastError;
 }
 
 function validateObject_object(obj, schema) {
@@ -1235,13 +1317,15 @@ function validateObject_object(obj, schema) {
     validateObject(obj[prop], schema.properties[prop]);
   }
   // Now check that the object doesn't have any properties not in the schema.
-  for (let prop in obj) {
-    if (!(prop in schema.properties)) {
-      throw validationErr(
-        "Object has property " + prop + " not in schema",
-        obj,
-        schema
-      );
+  if (!schema.additionalProperties) {
+    for (let prop in obj) {
+      if (!(prop in schema.properties)) {
+        throw validationErr(
+          "Object has property " + prop + " not in schema",
+          obj,
+          schema
+        );
+      }
     }
   }
 }
@@ -1254,9 +1338,9 @@ function validateObject_array(array, schema) {
   array.forEach(elt => validateObject(elt, schema.items));
 }
 
-function validateObject_string(str, schema) {}
-function validateObject_boolean(bool, schema) {}
-function validateObject_number(num, schema) {}
+function validateObject_string() {}
+function validateObject_boolean() {}
+function validateObject_number() {}
 
 function validationErr(msg, obj, schema) {
   return new Error(
